package ExSite::Dialog;
use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Input;

sub new ($%) {
	my ($this,%opt) = @_;
	my $obj = {};
	my $class = ref($this) || $this;
	bless $obj, $class;
	return $obj;
}

######-----------------------------------------------------------
# Default max image dimension
sub default_image_dimension{
	my ($this,$value) = @_;
	# Default image dimension
	my $default = 150;
	if (!$value){
		# Return default image dimension
		return $this->{default_dim} if $this->{default_dim};
		# Set default image dimension if there isn't one exists
		$this->{default_dim} = $default;
		return $this->{default_dim};
	}
	# Set new image dimension
	$this->{default_dim} = $value;
	return $this->{default_dim};
}
# Fetch pages
sub fetch_page{
	my ($this,$query) = @_;
	my $sectionid;
	if ($query->{section_id}){
		# Found the section id from the query string
		$sectionid = $query->{section_id};
	}else{
		if (exists $query->{data_id}){
			# Figure out the section id by using the content data object
			require ExSite::ContentData;
			my $cd = new ExSite::ContentData(id=>$query->{data_id});
			$cd->set_context();
			$sectionid = $cd->id("section");
		}else{
			# Figure out the section id by using the Page object
			require ExSite::Page;
			my $cd = new ExSite::Page(id=>$query->{cont_id});
			$cd = $cd->get();
			$sectionid = $cd->{section_id};
		}
	}
	# Fetch pages
	my @pages = $share{DB}->fetch_match("page",{section_id=>$sectionid,type=>"page"},'title');
	return \@pages;
}
# Figure out section id
sub find_section_id{
	my ($this,$query) = @_;
	my $cdata;
	if (exists($query->{data_id})){
		require ExSite::ContentData;
		$cdata = new ExSite::ContentData(id=>$query->{data_id});
	}else{
		require ExSite::Page;
		$cdata = new ExSite::Page(id=>$query->{cont_id});
	}
	$cdata->set_context();
	return $cdata->get("section","section_id");
}
# Adjust the size of the image
sub adjust_image_size{
	# $image_tag: html image tag
	# $fdata: encoded image data
	my ($this,$image_tag,$image) = @_;
	return "" if !$image;
	my $defaultimagedim = $this->default_image_dimension();
	my ($width,$height) = $image->dim();
	# Don't resize the image if its length and width are less or equal to 200
	return $image_tag if (($width <= $defaultimagedim) && ($height <= $defaultimagedim));
	if ($width >= $height){
		# Set width to 200 and adjust the height using the width ratio(width / default image dimension($defaultimagedim))
		$height = $this->shrink_image_dimension($width,$height);
		$width = $defaultimagedim;
	}else{
		# Set height to 200 and adjust the width using the height ratio(height / default image dimension($defaultimagedim))
		$width = $this->shrink_image_dimension($height,$width);
		$height = $defaultimagedim;
	}
	my $new_tag = $image_tag;
	# Remove existing height attribute
	$new_tag =~ s/height=.+?(\s|>)//gi;
	# Remove existing width attribute
	$new_tag =~ s/width=.+?(\s|>)//gi;
	# Remove the '>' in the html image tag
	$new_tag =~ s/>$//g;
	$new_tag .= " height=\"$height\" width=\"$width\">";
	return $new_tag;
}
# Shrink image dimension
sub shrink_image_dimension{
	my ($this,$reference_size,$need_adjustment) = @_;
	# Default image dimension
	my $defaultimagedim = $this->default_image_dimension();
	# Height/width or width/height ratio
	my $reduce_ratio = $reference_size / $defaultimagedim;
	my $final_size = $need_adjustment / $reduce_ratio;
	return int($final_size);
}
# Get object filename
sub get_object_filename{
	# $id: content id
	my ($this,$id) = @_;
	return "" if !$id;
	require ExSite::Content;
	my $c = new ExSite::Content(id=>$id);
	return $c->filename();
}
# Get library object
sub get_object{
	# $type: object type(either an image or an url)
	# $id: content id
	my ($this,$type,$id) = @_;
	return "" if !$id;
	require ExSite::Content;
	# Create a content object
	my $c = new ExSite::Content(id=>$id);
	$c->set_context("child");
	my $link = $c->get_html();
	my $out = "content-type: text/html\n\n";
	my $description = $c->get_my("description");
	if ($type eq "image"){
		my $file_type = $c->content_type();
		# only show previews for images and downloadable documents
		if ($file_type =~ /^(image|application)/i){
			if ($file_type =~ /^image/i){
				$out .= $this->adjust_image_size($c->get_html(undef,$c->get_url("dynamic")),$c->get_image());
			}else{
				$out .= "<a href=\"".$c->get_url()."\" target=\"_blank\">".$c->filename()."</a>";
			}
		}else{
			$out .= $description || "No preview available";
		}
	}elsif ($type eq "url"){
		if ($link =~ /img/i){
			$out .= $c->get_url("dynamic");
		}else{
			$out .= "$config{server}->{HTMLpath}/_ExSite/images/checker.png";
		}
	}
	return $out;
}
# Fetch libraries
sub fetch_library{
	# $sectionid: section id
	my ($this,$sectionid) = @_;
	return [] if !$sectionid;
	require ExSite::Section;
	my $s = new ExSite::Section(id=>$sectionid);
	# fetch libraries
	my @records = $s->fetch_libraries;
	return \@records;
}
# Is this a file in the library?
sub is_exist{
	# $sectionid: section id
	# $name: name of the object
	my ($this,$sectionid,$name) = @_;
	return 0 if !$sectionid || !$name;
	# Fetch section information
	require ExSite::Section;
	# Create a section object
	my $section = new ExSite::Section(id=>$sectionid);
	# Set context
	$section->set_context();
	# Search for the file
	my $c = $section->find($name);
	return 0 if !$c;
	return 1 if $c->id();
	return 0;
}
# Truncate text
sub truncate_text{
	# $text: text
	# $num: character limit
	my ($this,$text,$num) = @_;
	my $default = 150;
	my $cutoff = $num || $default;
	# Truncate the text
	if (length($text) > $cutoff){
		$text = substr($text,0,$cutoff);
		$text .= "...";
	}
	return $text;
}
######-----------------------------------------------------------
sub run{
	my $i = new ExSite::Input;
	my $ml = &get_obj("ML");
	if ($share{DB}->authorize() < $config{form}{tinymce}{dialog}{access}){
		return "content-type: text/html\n\n".$ml->html($ml->head().$ml->body($ml->p("Access denied.")));
	}
	my $path = $i->path_info();
	if ($path =~ /Image/){
		return ExSite::Dialog::Image->new()->run($i);
	}elsif ($path =~ /Link/){
		return ExSite::Dialog::Link->new()->run($i);
	}elsif ($path =~ /Dyn/){
		return ExSite::Dialog::DynamicContent->new()->run($i);
	}
	return "content-type: text/html\n\n".$ml->html($ml->head().$ml->body($ml->p("Unknown Options")));
}
#########################################
package ExSite::Dialog::TinyMCE;
use strict;
use ExSite::Config;
sub new ($%) {
	my ($this,%opt) = @_;
	my $obj = {};
	my $class = ref($this) || $this;
	bless $obj, $class;
	return $obj;
}
sub conf{
	my ($this,$base,$new) = @_;
	if (ref ($new) =~ /array/i ){
		for(my $i = 0; $i < scalar(@$base); $i++){
			delete $base->[$i];
		}
		for(my $i = 0; $i < scalar(@$new); $i++){
			if (ref($new->[$i]) =~ /array|hash/i){
				$this->conf($base->[$i],$new->[$i]);
			}else{
				$base->[$i] = $new->[$i];
			}
		}
	}elsif (ref($new) =~ /hash/i){
		foreach my $key (keys(%$new)){
			if (ref($new->{$key}) =~ /array|hash/i){
				$this->conf($base->{$key},$new->{$key});
			}else{
				$base->{$key} = $new->{$key};
			}
		}
	}
	return;
}
sub css{
	my $ml = ExSite::Config::get_obj("ML");
	return $ml->style(".mce-menubtn span, div.mce-menubtn, .mce-btn button {line-height: 16px !important;}");
}
sub jq_detect{
	return <<END;
var _stat_mce_load_jquery=false;function _mce_load_jquery(){if(typeof(jQuery)=='undefined'){if(!_stat_mce_load_jquery){_stat_mce_load_jquery=true;document.write("<scrip"+"t src='$config{jquery}'></scrip"+"t>");}setTimeout("_mce_load_jquery()",50);}};_mce_load_jquery();
END
}
sub setting{
	my ($this,$custom_opt,$section_id) = @_;
	my @disable_cmd;
	my %econfig = %{$config{form}{tinymce}};
	$this->conf(\%econfig,$custom_opt->{editor}{tinymce}) if ref($custom_opt->{editor}{tinymce}) =~ /hash/i;
	my %editor_setting =  %{$econfig{editor}};
	$editor_setting{script_url} = $config{server}{server}.$config{server}{HTMLpath}.$config{form}{tinymce}{path}."/tiny_mce.js";

	$editor_setting{height} = $custom_opt->{editor_height} if $custom_opt->{editor_height};
	$editor_setting{width} = $custom_opt->{editor_width} if $custom_opt->{editor_width};
	my $editor_mode = exists $econfig{mode}{toolbar}{$econfig{mode}{current}} ? $econfig{mode}{current}  : $econfig{mode}{default};

	$editor_setting{menu} = $econfig{mode}{toolbar}{$editor_mode}{menu};
	$editor_setting{menubar} = $econfig{mode}{toolbar}{$editor_mode}{menubar};

	# Link dialog
	my $dlglink_advanced = $editor_mode eq "basic" ? 2 :  ($custom_opt->{editor}{op_link}  || $config{form}{htmleditor}{op_link}) == 2 ? 2 : !$section_id ? 2 : 1;
	$editor_setting{plugin_link}{url} = "$config{server}{server}$config{server}{CGIpath}/$econfig{dialog}{dlglink}{script}?mode=${dlglink_advanced}".($section_id ? "&section_id=${section_id}" : "");
	$editor_setting{plugin_link}{width} = $econfig{dialog}{dlglink}{popup}{($dlglink_advanced == 1 ? "full" : "basic")}{width};
	$editor_setting{plugin_link}{height} = $econfig{dialog}{dlglink}{popup}{($dlglink_advanced == 1 ? "full" : "basic")}{height};

	# Image dialog
	my $dlgimg_advanced = $editor_mode eq "basic" ? 0 : $section_id ? 1 : 0;
	$editor_setting{plugins} .= ",dlgimg";
	$editor_setting{plugin_image}{url} = "$config{server}{server}$config{server}{CGIpath}/$econfig{dialog}{dlgimg}{script}?".($dlgimg_advanced ? "section_id=$section_id" : "");
	$editor_setting{plugin_image}{width} = $econfig{dialog}{dlgimg}{popup}{($dlgimg_advanced ? "full" : "basic")}{width};
	$editor_setting{plugin_image}{height} = $econfig{dialog}{dlgimg}{popup}{($dlgimg_advanced ? "full" : "basic")}{height};
	$editor_setting{plugin_image}{ajx_url} = "$config{server}{server}$config{server}{CGIpath}/$econfig{dialog}{dlgimg}{script}";
	$editor_setting{plugin_image}{section_id} = $section_id;

	# Dynamic content dialog
	$editor_setting{plugin_dyncontent}{img} = "$config{server}{server}$config{server}{HTMLpath}$config{form}{tinymce}{path}/plugins/dlgdyn/icon.gif";
	if ($editor_mode ne "basic"){
		if ($section_id){
			$editor_setting{plugins} .= ",dlgdyn";
			$editor_setting{plugin_dyncontent}{url} = "$config{server}{server}$config{server}{CGIpath}/$econfig{dialog}{dlgdyn}{script}?section=$section_id";

			$editor_setting{plugin_dyncontent}{ajx_url} = "$config{server}{server}$config{server}{CGIpath}/$econfig{dialog}{dlgdyn}{script}";
			$editor_setting{plugin_dyncontent}{section_id} = $section_id;
		}
	}

	# Mailto dialog
	$editor_setting{plugins} .= ",dlgmailto";
	$editor_setting{plugin_mailto}{img} = "$config{server}{server}$config{server}{HTMLpath}$config{form}{tinymce}{path}/plugins/dlgmailto/icon.gif";
	my $icon_mailto = "$config{server}{HTMLpath}/_ExSite/_Modules/icon.gif";
	require ExSite::Module;
	my $mod = ExSite::Module::get_module("MailTo");
	if ($mod){
		$icon_mailto = $mod->icon($config{iconset});
	}
	$editor_setting{plugin_mailto}{plugin_img} = $icon_mailto;

	# Font drop down
	if ($econfig{editor}{font_formats}){
		$editor_setting{font_formats} = $econfig{editor}{font_formats};
	}

	# Editor toolbar
	for (my $i = 1;$i <= 4;$i++){
		$editor_setting{"toolbar".$i} = (ref $econfig{mode}{toolbar}{$editor_mode}{"toolbar_row".$i} !~ /array/i) ? "" : join(",|,", map {$econfig{group}{$_} || ()} @{$econfig{mode}{toolbar}{$editor_mode}{"toolbar_row".$i}});
	}
	if ($editor_mode ne "basic"){
		$editor_setting{plugins} .= ",advlist";
	}

	# widget
	if ($econfig{widget}){
		foreach my $widget(keys %{$econfig{widget}}){
			next if !$econfig{widget}{$widget};
			$editor_setting{widget}{$widget} = $econfig{widget}{$widget};
		}
	}
	return {conf=>\%econfig,init=>\%editor_setting};
}

#########################################
package ExSite::Dialog::Base;
use strict;
use ExSite::Config;
use ExSite::Section;
sub new ($%) {
	my ($this,%opt) = @_;
	my $obj = {};
	my $class = ref($this) || $this;
	bless $obj, $class;
	return $obj;
}
sub DB{
	return $share{DB};
}
sub ML{
	return ExSite::Config::get_obj("ML");
}
sub run{
	return;
}
sub content{
	my $this = shift;
	my $out = shift;
	return  "Expires: -1\nCache-Control: no-cache\nCache-Control: max-page=0, no-cache, no-store, must-revalidate\nPragma: no-cache\nContent-type: text/html\n\n".$out;
}
# Fetch libraries
sub fetch_library{
	# $sectionid: section id
	my ($this,$sectionid) = @_;
	return [] if !$sectionid;
	my $s = new ExSite::Section(id=>$sectionid);
	# fetch libraries
	my @records = $s->fetch_libraries;
	return \@records;
}
# Truncate text
sub truncate_text{
	# $text: text
	# $num: character limit
	my ($this,$text,$num) = @_;
	my $default = 150;
	my $cutoff = $num || $default;
	# Truncate the text
	if (length($text) > $cutoff){
		$text = substr($text,0,$cutoff);
		$text .= "...";
	}
	return $text;
}
#########################################
package ExSite::Dialog::DynamicContent;
use strict;
use ExSite::Config;
use ExSite::Module qw(&get_module &my_module_list);
use ExSite::Misc;
use JSON;

use ExSite::Dialog;
use base qw(ExSite::Dialog::Base);


sub run{
	my ($this,$INPUT) = @_;
	my $input = $INPUT->combine();
	my $ml = $this->ML();
	if ($input->{action} eq "geturl"){
		# icon url
		my $mod = ExSite::Module::get_module($input->{name});
		return "$config{server}{HTMLpath}/_ExSite/_Modules/icon.gif" if !$mod;
		return $this->content($mod->icon($config{iconset}));
	# get module parameters
	}elsif ($input->{action} eq "getparam"){
		# Running DynContentParameter routine to create
		# the optional parameter input field for the dialog
		#
		# - the name of the input field has to be 'param'.
		#

#		my $default = $ml->table($ml->tr($ml->td($ml->strong("Parameter (optional)")).$ml->td($ml->input(undef,{type=>"text",name=>"param",value=>""}))));

		my $default = $ml->strong("Parameter (optional):").$ml->input(undef,{type=>"text",name=>"param",value=>""});
		return $this->content($default) if (!$input->{name} || !$input->{section});
		my $mod = ExSite::Module::get_module($input->{name});
		return $this->content(($mod ? $mod->ioctl("DynContentParameter",$input->{section},$input->{param})  || $default : $default));
	}
	return $this->content($ml->html($ml->head($ml->title("Web Application")).$ml->body($this->form($input->{section}))));
}



sub script{
	return "$config{server}{server}$config{server}{CGIpath}/$config{form}{tinymce}{dialog}{dlgdyn}{script}";
}
sub form{
	my ($this,$sectionid) = @_;
	my $ml = $this->ML();
	my @list;
	my $blacklist = $config{dcd}{blacklist};
	my @modlist = defined $config{dcd}{whitelist} ? @{$config{dcd}{whitelist}} : &ExSite::Module::my_module_list($sectionid,$blacklist);
	foreach my $mod (@modlist) {
	    my $name = exists $config{webtop}{module_name}{$mod} ? $config{webtop}{module_name}{$mod} : $mod;
	    next if (! $name);
	    push @list, $ml->option($name,{value=>$mod});
	}
	return $ml->p("No web application found.") if (scalar(@list) < 1);
	my %tdata = (
		tinymce_path=>"$config{server}{server}$config{server}{HTMLpath}$config{form}{tinymce}{path}",
		module_list=>join("",@list),
		dlg_file=>$this->script(),
		section_id=>$sectionid,
		cgipath=>"$config{server}{CGIpath}",
		jquery=>"$config{jquery}",
		
	);

	my $template = <<END;
<!DOCTYPE html>
<html lang="en">
<head>
<meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no">
<link rel="stylesheet" href="$config{server}{server}$config{server}{HTMLpath}$config{form}{tinymce}{path}/skins/lightgray/skin.min.css">
</head>
<body>

<script src="[[jquery]]"></script>

<!-- Latest compiled and minified CSS -->
<link rel="stylesheet" href="//netdna.bootstrapcdn.com/bootstrap/3.0.2/css/bootstrap.min.css">

<!-- Optional theme -->
<link rel="stylesheet" href="//netdna.bootstrapcdn.com/bootstrap/3.0.2/css/bootstrap-theme.min.css">

<!-- Latest compiled and minified JavaScript -->
<script src="//netdna.bootstrapcdn.com/bootstrap/3.0.2/js/bootstrap.min.js"></script>


<script>
var dyndlg = {
	closeFlag : 0,
	close : function(){
		this.closeFlag = 1;
	},
	getDialogParam : function(id,module,param) {
		if (module == '') return;
		var h,w;
		jQuery.ajax({'url':'[[dlg_file]]','type':'GET','dataType':'html','async':false,'success':function(data){
			jQuery("#" + id).html(data);
			jQuery("#" + id).find("script").each(function(){
				eval(jQuery(this).text());
			});
		},'data':{'action':'getparam','name':module,'section':'[[section_id]]','section_id':'[[section_id]]','param':escape(param)}});
		h = jQuery("form#dyndlg").height() + 40;
		w = jQuery("form#dyndlg").width() + 15;
	},
	insert : function(){
		if (this.closeFlag){
			(parent.tinymce.activeEditor || top.tinymce.activeEditor).windowManager.close();
			return;
		}
		var ed = parent.tinymce.activeEditor, n = jQuery("#dyncontent").val(), p = '', u;
 		jQuery("form#dyndlg :input").each(function(){
			var _i = jQuery(this).attr('name'), _v = new String(), _t = jQuery(this).attr('type');
			if ( _t == 'radio' || _t == 'checkbox'){
				if (jQuery(this).is(":checked")){
					_v = jQuery(this).val();
				}else{
					_v = '';
				}
			}else{
				_v = jQuery(this).val();
			}
			if (_i != 'dyncontent' && _i != 'insert' && _i != 'cancel' && _v){
				if (p) p += '&amp;';
				if (_i != 'param') p += _i + '=';
				_v = _v.replace(/(\\r\\n|\\n|\\r)/gm," ");
				p += _v;
			}
		});
		if (!n){
			alert('You must select a web application');
			return;
		}
		jQuery.ajax({'url':'[[dlg_file]]','type':'GET',data:{'action':'geturl','name':n},'dataType':'html','async':false,'success':function(data){u = data;}});
		m = '<img alt="' + n + '(' + p + ')" name="' + n + '" src="' + u + '" title="' + n + '" border="3" style="border-style:outset;">';
		ed.execCommand("mceInsertContent", false, m);
		(parent.tinymce.activeEditor || top.tinymce.activeEditor).windowManager.close();
	},
	init : function(){
		this.closeFlag = 0;
		var inst = parent.tinymce.activeEditor,elm;
		elm = inst.selection.getNode();
		elm = inst.dom.getParent(elm,'IMG');
		if (elm != null && elm.nodeName == 'IMG'){
			var n,p,t = new String(elm.getAttribute('alt'));
			t.replace(/\\n/gi," ");
			if (t.match(/\\w+\\(.*\\)/gi)){
				t.replace(/(\\w+)\\((.*)\\)/gi,function(m,f,s){n=f;p=s;});
			}
			if (n){
				jQuery("#dyncontent").val(n);
				dyndlg.getDialogParam('param_input',n,p)
			}
		}
	},
	// optional: overload this function for module specific form processing
	pre_insert : function(){
		return;
	}
};
</script>
<div class="container">
	<form id="dyndlg" onsubmit="dyndlg.pre_insert();dyndlg.insert();return false;" style="display:inline-block;" role="form">
	<div class="form-group">
		<label class="control-label" for="dyncontent">Web Application</label>
		<select class="form-control" id="dyncontent" name="dyncontent" onChange="dyndlg.getDialogParam('param_input',this.value)">
			<option value="">== Select ==</option>
			[[module_list]]
		</select>
	</div>

	<div class="form-group">
		<label class="control-label">Options</label>
		<div id="param_input" class="form-control-static">
			<strong>Parameter (optional):</strong><input type="text" name="param" value="">
		</div>
	</div>
	<div class="form-group text-right">
		<button class="btn mce-btn mce-primary">Insert</button>
		<button class="btn btn-default active" onclick="dyndlg.close();">Cancel</button>
	</div>
	</form>
</div>
</body>
</html>
END
	return ExSite::Misc::substitute($template,\%tdata);
}

#########################################
package ExSite::Dialog::Image;
use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Image;
use ExSite::Content;
use MIME::Base64;

use ExSite::Dialog;
use base qw(ExSite::Dialog::Base);

sub run{
	my ($this,$INPUT) = @_;
	my $input = $INPUT->combine();
	my $ml = $this->ML();
	if ($input->{action} eq "getimagelist"){
		# get image/document list
		return $this->getImageList($input);
	}elsif ($input->{action} eq "getimage"){
		# get single image/document
		return $this->getImage($input);
	}elsif ($input->{action} eq "geturl"){
		# get img url
		return $this->getUrl($input);
	}
	# process image/document upload
	return $this->content($ml->html($ml->head($ml->title("Image/Document Tool")).$ml->body(($input->{action} eq "upload_file" ? $this->upload($input) : "").$this->form($input))));
}

# javascript popup
sub alertPopup{
	my ($this,$msg) = @_;
	return "<script>alert('$msg');</script>";
}
# process form data
sub upload{
	my $this = shift;
	my $data = shift;
	return $this->alertPopup("Error: File upload failed.  No library is selected.") if !$data->{library};
	return $this->alertPopup("Error: File upload failed.  No file is selected.") if !$data->{fileupload};
	my $filename;
	my $rawdata = undef;
	my $size = 0;
	my $mimedata = "";
	my $file = $data->{fileupload};
	my $continue = 1;
	# read upload data
	while ($continue){
		$continue = read($file,$rawdata,1024,$size);
		$size += $continue;
	}
	return $this->alertPopup("Error: File upload failed.  No file data was received.") if !$size;
	my @path = split /[:\\\/]/, $file;
	if ($path[-1]){
		$filename = ExSite::Misc::clean_filename($path[-1]);
	}
	my $fdata;
	if (ExSite::Misc::MimeType($filename) !~ /^image/i){
		$fdata = $filename."#".MIME::Base64::encode_base64($rawdata);
	}else{
		my $img = new ExSite::Image($filename,$rawdata);
		$img->shrink(dim=>640);
		$fdata = $img->encode;
	}
	my $content = new ExSite::Content();
	# insert form data
	my $cid = $content->make_content({name=>$filename,page_id=>$data->{library}},{fdata=>$fdata});
	if ($cid){
		return $this->alertPopup("$filename has been uploaded successfully.");
	}else{
		my @errors = $content->fetch_diagnostics("error");
		my $error_message = join("\\n",@errors);
		return $this->alertPopup($error_message);
	}
	return;
}
# create an image/document list
sub getImageList{
	my ($this,$data) = @_;
	my $ml = $this->ML();
	my $db = $this->DB();
	my %page = $db->fetch("page",$data->{page_id});
	my $list = $db->fetch_match("content",{page_id=>$data->{page_id}});
	my $selector;
	my $limit = 25;
	foreach my $l (@$list){
		my $name = "$l->{name}";
		my $ext = "";
		if ($name =~ /(.+)\.(.+)$/){
			$name = $1;
			$ext = $2;
		}
		my $filename = (length($name) > $limit ? substr($name,0,$limit)."..." : $name).($ext ? ".${ext}" : "");
		$selector .= $ml->option($filename,{value=>sprintf("%s:%s/%s",$l->{content_id},$page{filename},$l->{name})});
	}
	return $this->content($ml->select($ml->option("== Select ==",{value=>""}).$selector,{id=>"obj_id",name=>"obj_id",onChange=>"dlgimg.showimage()",class=>"form-control"}));
}
sub getImage{
	my ($this,$data) = @_;
	return $this->get_object("image",$data->{content_id});
}
sub getUrl{
	my ($this,$data) = @_;
	return $this->get_object("url",$data->{content_id});
}
# Get library object
sub get_object{
	# $type: object type(either an image or an url)
	# $id: content id
	my ($this,$type,$id) = @_;
	return "" if !$id;
	my $ml = $this->ML();
	# Create a content object
	my $c = new ExSite::Content(id=>$id);
	$c->set_context("child");
	my $link = $c->get_html();
	my $out = "content-type: text/html\n\n";
	my $description = $c->get_my("description");
	if ($type eq "image"){
		my $file_type = $c->content_type();
		# only show previews for images and downloadable documents
		if ($file_type =~ /^(image|application)/i){
			if ($file_type =~ /^image/i){
				$out .= $this->adjust_image_size($c->get_html(undef,$c->get_url("dynamic")),$c->get_image());
			}else{
				$out .= $ml->a($c->filename(),{href=>$c->get_url(),target=>"_blank"});
			}
		}else{
			$out .= $description || "No preview available";
		}
	}elsif ($type eq "url"){
		if ($link =~ /img/i){
			my %page = $share{DB}->fetch("page",$c->get_my("page_id"));
			$out .= $c->get_url("dynamic",$page{filename}."/".$c->get_my("name"));
		}else{
			$out .= "$config{server}{HTMLpath}/_ExSite/images/checker.png";
		}
	}
	return $out;
}
sub adjust_image_size{
	# $image_tag: html image tag
	# $fdata: encoded image data
	my ($this,$image_tag,$image) = @_;
	return "" if !$image;
	my $defaultimagedim = $this->default_image_dimension();
	my ($width,$height) = $image->dim();
	# Don't resize the image if its length and width are less or equal to 200
	return $image_tag if (($width <= $defaultimagedim) && ($height <= $defaultimagedim));
	if ($width >= $height){
		# Set width to 200 and adjust the height using the width ratio(width / default image dimension($defaultimagedim))
		$height = $this->shrink_image_dimension($width,$height);
		$width = $defaultimagedim;
	}else{
		# Set height to 200 and adjust the width using the height ratio(height / default image dimension($defaultimagedim))
		$width = $this->shrink_image_dimension($height,$width);
		$height = $defaultimagedim;
	}
	my $new_tag = $image_tag;
	# Remove existing height attribute
	$new_tag =~ s/height=.+?(\s|>)//gi;
	# Remove existing width attribute
	$new_tag =~ s/width=.+?(\s|>)//gi;
	# Remove the '>' in the html image tag
	$new_tag =~ s/>$//g;
	$new_tag .= " height=\"$height\" width=\"$width\">";
	return $new_tag;
}
# Default max image dimension
sub default_image_dimension{
	my ($this,$value) = @_;
	# Default image dimension
	my $default = 100;
	if (!$value){
		# Return default image dimension
		return $this->{default_dim} if $this->{default_dim};
		# Set default image dimension if there isn't one exists
		$this->{default_dim} = $default;
		return $this->{default_dim};
	}
	# Set new image dimension
	$this->{default_dim} = $value;
	return $this->{default_dim};
}
# Shrink image dimension
sub shrink_image_dimension{
	my ($this,$reference_size,$need_adjustment) = @_;
	# Default image dimension
	my $defaultimagedim = $this->default_image_dimension();
	# Height/width or width/height ratio
	my $reduce_ratio = $reference_size / $defaultimagedim;
	my $final_size = $need_adjustment / $reduce_ratio;
	return int($final_size);
}
sub script{
	return "$config{server}{server}$config{server}{CGIpath}/$config{form}{tinymce}{dialog}{dlgimg}{script}";
}
sub form{
	my $this = shift;
	my $data = shift;
	my $db = $this->DB();
	my $ml = $this->ML();
	my $selectedlib = 0;
	my $sectionid = $data->{section_id};
	my $libs = $this->fetch_library($sectionid);
	my $allow_upload = $sectionid && $config{form}{tinymce}{dialog}{dlgimg}{allow_file_upload} ? 1 : 0;
	my %tdata = (
		dlg_file=>$this->script(),
		tinymce_path=>"$config{server}{server}$config{server}{HTMLpath}$config{form}{tinymce}{path}",
		jquery_js=>$config{jquery},
		insert_library_select=>join("",map {my %opt = (value=>$_->{page_id});if ($_->{filename} =~ /image/i && !$selectedlib){$selectedlib = $_->{page_id};$opt{selected} = undef;} $ml->option($_->{title},\%opt)} @$libs),
		selected_lib=>$selectedlib,
		insert_image_attribute=>join("",map {$ml->option($_->{label},{value=>$_->{value}})} (({label=>"Left",value=>"left"},{label=>"Right",value=>"right"}), ($config{form}{htmleditor}{imgalign_all} ? ({label=>"Top",value=>"top"},{label=>"Bottom",value=>"bottom"}) : ()))),
		allow_file_upload=>($allow_upload ? 1 : undef),
		upload_library_select=>($allow_upload ? join("",map {$db->owns($_->{section_id}) ? $ml->option($_->{title},{value=>$_->{page_id}}): ()} @$libs) : ""),
		section_id=>($sectionid || 0),
		advance_mode=>($sectionid ? 1 : undef),
		https=>($config{server}{protocol} =~ /^https$/i ? 1 : undef)
	);
	my $template = <<END;
<!DOCTYPE html>
<html lang="en">
<head>
<meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no">
</head>
<body>
<script src="[[jquery_js]]"></script>

<!-- Latest compiled and minified CSS -->
<link rel="stylesheet" href="//netdna.bootstrapcdn.com/bootstrap/3.0.2/css/bootstrap.min.css">
<!-- Optional theme -->
<link rel="stylesheet" href="//netdna.bootstrapcdn.com/bootstrap/3.0.2/css/bootstrap-theme.min.css">
<!-- Latest compiled and minified JavaScript -->
<script src="//netdna.bootstrapcdn.com/bootstrap/3.0.2/js/bootstrap.min.js"></script>

<link rel="stylesheet" href="$config{server}{server}$config{server}{HTMLpath}$config{form}{tinymce}{path}/skins/lightgray/skin.min.css">
<style>
.numeric-field {
width: 100px !important;
}
</style>

<script>
var dlgimg = {
	closeFlag : 0,
	close : function(){
		this.closeFlag = 1;
	},
[[include:advance_mode]]
	isImage : 0,
	showimage : function(){
		var id = jQuery("#obj_id").val();
		if (!id){
			return;
		}
		jQuery.ajax({'url':'[[dlg_file]]','type':'GET','dataType':'html','async':false,'success':function(data){var re = RegExp('img');if (re.test(data)){dlgimg.showAlignmentForm();}else{dlgimg.hideAlignmentForm();}jQuery("#disp").html(data);},'data':{'action':'getimage','content_id':id}});
	},
	getImageList : function(){
		var pageid = jQuery("#lib_id").val();
		this.hideAlignmentForm();
		this.clearPreviewBlock();
		jQuery.ajax({'url':'[[dlg_file]]','type':'GET','dataType':'html','async':false,'success':function(data){jQuery("#imageSelector").html(data);},'data':{'action':'getimagelist','page_id':pageid}});
	},
	elmAlignment : [
		{"name":"imagealignment","type":"disabled"},
		{"name":"imagenoncontent","type":"disabled"},
		{"name":"imageborderwidth","type":"disabled"},
		{"name":"imagenoncontent","type":"checked"},
		{"name":"imagedesc","type":"disabled"},
		{"name":"imageheight","type":"disabled"},
		{"name":"imagewidth","type":"disabled"}
	],
	hideAlignmentForm : function(){
		for(var i = 0;i < this.elmAlignment.length;i++){
			jQuery("#" + this.elmAlignment[i].name).attr(this.elmAlignment[i].type,true);
		}
		this.isImage = 0;
	},
	showAlignmentForm : function(){
		for(var i = 0;i < this.elmAlignment.length;i++){
			jQuery("#" + this.elmAlignment[i].name).attr(this.elmAlignment[i].type,false);
		}
		this.isImage = 1;
	},
	clearPreviewBlock : function(){
		jQuery("#disp").html('');
	},
	getContentID : function(val){
		var stringsplt = val.split(':');
		return stringsplt[0];
	},
	getContentName : function(val){
		var result,stringsplt;
		result = '';
		stringsplt = val.split(':');
		for(var i = 1; i < stringsplt.length; i++){
			result += stringsplt[i];
		}
		return result;
	},
	inputImgBorderWidth : function(obj){
		var elms = [
			{"name":"imageborderwidth","type":"disabled"},
			{"name":"imagealignment","type":"disabled"},
			{"name":"imagedesc","type":"disabled"},
			{"name":"imageheight","type":"disabled"},
			{"name":"imagewidth","type":"disabled"}
		];
		var st = obj.checked ? true : false;
		for(var i = 0; i < elms.length; i++){
			jQuery("#" + elms[i].name).attr(elms[i].type,st);
		}
	},
	msie : function(){
		var ua = window.navigator.userAgent;
		var ms_ie = ua.indexOf('MSIE ');
		if (ms_ie > 0) {
			// IE 10 or older => return version number
			return parseInt(ua.substring(ms_ie + 5, ua.indexOf('.', ms_ie)), 10);
		}
		return 0;
	},
	insertImgDoc : function(){
		if (this.closeFlag){
			(parent.tinymce.activeEditor || top.tinymce.activeEditor).windowManager.close();
			return;
		}
		var oid = jQuery("#obj_id").val(); p_name = this.getContentName(oid);
		if (!p_name){
			alert('You must select an image/document');
			return;
		}
		var objId = this.getContentID(oid);
		jQuery.ajax({'url':'[[dlg_file]]','type':'GET','dataType':'html','async':false,'success':function(data){if (dlgimg.msie()){jQuery('<img>').attr('src',data).ready(function(){dlgimg.doInsertImgDoc(p_name,data,this);});}else{jQuery('<img>').attr('src',data).load(function(){dlgimg.doInsertImgDoc(p_name,data,this);});}},'data':{'action':'geturl','content_id':objId}});
	},
	doInsertImgDoc : function(p_name,p_url,img){
		if (this.closeFlag){
			(parent.tinymce.activeEditor || top.tinymce.activeEditor).windowManager.close();
			return;
		}
		var ed = parent.tinymce.activeEditor,p_align,non_img,p_borderwidth,p_desc,p_width,p_height,p_alt,p;
		p_align = jQuery("#imagealignment").val();
		non_img = jQuery("#nonimagecontent").is(":checked") ? 1 : 0;
		p_borderwidth = jQuery("#imageborderwidth").val() || 0;
		p_desc = new String(jQuery("#imagedesc").val() || '').replace("'", "\\\'").replace('"',"&quot;").replace("<","&lt;").replace(">","&gt;");
		var re = new RegExp('checker');
		if (re.test(p_url)){
			img.width = 100;
			img.height = 50;
		}
		p_width = jQuery("#imagewidth").val() || '';
		p_height = jQuery("#imageheight").val() || '';
		p_alt = 'content(' + p_name + ')';
		if (dlgimg.isImage > 0 && non_img < 1){
			p_alt = '[' + '[' + p_name + ']' + ']';
		}
		p = '<img alt="' + p_alt + '" src="' + p_url + '" title="' + (p_desc == '' ? p_name : p_desc) + '"' + (p_width ? ' width="' + p_width + '"' : '') + (p_height ? ' height="' + p_height + '"' : '');
		p += p_align != '' ? ' class="' + (p_align == 'left' ? 'alignLeft' : 'alignRight' ) + '" align="' + p_align + '"' : '';
		p += (!isNaN(p_borderwidth) ? ' border="' + p_borderwidth + '"' : '') + '>';
		ed.execCommand("mceInsertRawHTML", false, p);
		(parent.tinymce.activeEditor || top.tinymce.activeEditor).windowManager.close();
	},
[[/include:advance_mode]]
[[include:allow_file_upload]]
	insertFile : function() {
		if (this.closeFlag){
			(parent.tinymce.activeEditor || top.tinymce.activeEditor).windowManager.close();
			return;
		}
		var err = new Array();
		if (!jQuery("#library").val())
			err.push('No library selected');
		if (!jQuery("#fileupload").val())
			err.push('No file selected');
		if (err.length > 0){
			alert("Error:\\n" + err.join("\\n"));
			return false;
		}
		return true;
	},
[[/include:allow_file_upload]]
	insertUrl : function() {
		if (this.closeFlag){
			(parent.tinymce.activeEditor || top.tinymce.activeEditor).windowManager.close();
			return;
		}
		var p_url = jQuery("#img_url").val();
		if (!p_url){
			alert('You must enter an image url');
			return;
		}[[include:https]]
		if (! /^https:/.test(p_url)){
			alert('Your website uses secure URLs. Please enter a secure URL.');
			return;
		}[[/include:https]]
		var ed = parent.tinymce.activeEditor,tmpImg = new Image(),p_align = jQuery("#url_imagealignment").val(),p_borderwidth,p_desc,p_width,p_height,p;
		tmpImg.src = p_url;
		p_borderwidth = jQuery("#url_imageborderwidth").val() || 0;
		p_desc = new String(jQuery("#url_imagedesc").val() || '').replace("'", "\\\'").replace('"',"&quot;").replace("<","&lt;").replace(">","&gt;");
		p_width = jQuery("#url_imagewidth").val() || tmpImg.width;
		p_height = jQuery("#url_imageheight").val() || tmpImg.height;
		p = '<img src="' + p_url + '" title="' + p_desc + '"' + (p_width > 0 ? ' width="' + p_width + '"' : '') + (p_height > 0 ? ' height="' + p_height + '"' : '') + (typeof(p_align) != 'undefined' ? ' align="' + p_align + '" class="' + (p_align == 'left' ? 'alignLeft' : 'alignRight') + '"' : '') + (!isNaN(p_borderwidth) ? ' border="' + p_borderwidth + '"' : '') + '>';
		ed.execCommand("mceInsertContent", false, p);
		(parent.tinymce.activeEditor || top.tinymce.activeEditor).windowManager.close()
	}[[include:advance_mode]],
	init : function(){
		this.closeFlag = 0;
		jQuery("#lib_id").val([[selected_lib]]);
		this.getImageList();
		var input_style = {
			'background-color' : '#f0f0ee',
			'border' : 'none'
		};
		jQuery(":input[type='checkbox']").css(input_style);
	}
[[/include:advance_mode]]
};

\$(document).ready(function(){
[[include:advance_mode]]
	dlgimg.init();
[[/include:advance_mode]]
	jQuery("#img_prop_link").click(function(){
		if (jQuery("#img_prop").is(":visible")){
			jQuery("#img_prop").addClass("hide");
			jQuery("#img_toggle").removeClass("glyphicon-chevron-up");
			jQuery("#img_toggle").addClass("glyphicon-chevron-down");
		}else{
			jQuery("#img_prop").removeClass("hide");
			jQuery("#img_toggle").removeClass("glyphicon-chevron-down");
			jQuery("#img_toggle").addClass("glyphicon-chevron-up");
		}
	});
});
</script>
<div class="container mce-container-body mce-abs-layout">
<br>
[[include:advance_mode]]

<div id="content">
	<ul id="tabs" class="nav nav-tabs" data-tabs="tabs">
		<li class="active"><a href="#imgdoc" data-toggle="tab">Internal Image</a></li>
		<li><a href="#url" data-toggle="tab">External Image</a></li>
[[/include:advance_mode]]
[[include:allow_file_upload]]
		<li><a href="#upload" data-toggle="tab">File Upload</a></li>
[[/include:allow_file_upload]]
[[include:advance_mode]]
	</ul>
	<div id="my-tab-content" class="tab-content">
		<div class="tab-pane active" id="imgdoc">
			<div class="container">
			<form id="img_frm" onSubmit="dlgimg.insertImgDoc();return false;" action="#" role="form" class="form-horizontal">
				<div class="form-group">
					<label class="control-label" for="lib_id">Library</label>
						<select id="lib_id" name="lib_id" onChange="dlgimg.getImageList()" class="form-control"><option value="">== Select ==</option>[[insert_library_select]]</select>
				</div>
				<div class="form-group">
					<label class="control-label" for="obj_id">Image/Document</label>
					<span id="imageSelector"><select id="obj_id" name="obj_id" onChange="dlgimg.showimage()" class="form-control"><option value="">== Select ==</option></select></span>
				</div>
					<div class="form-group">
						<label class="control-label">Alignment</label>
						<select id="imagealignment" name="align" class="form-control"><option value="">None</option>[[insert_image_attribute]]</select>
					</div>
				<div class="form-group">
					<label>Image Properties</label> <a id="img_prop_link" style="cursor:pointer;"><span id="img_toggle" class="glyphicon glyphicon-chevron-down"></span></a>
				</div>

				<div id="img_prop" class="hide container">

					<div class="form-group">
						<label class="control-label" for="imagedesc">Description</label>
						<input type="text" id="imagedesc" name="imagedesc" value="" class="form-control">
					</div>
					<div class="form-group">
						<label class="control-label">Dimension</label>
						<div class="input-group">
							<input class="form-control numeric-field" type="text" id="imagewidth" name="imagewidth" value="" size="4">
							x
							<input class="form-control numeric-field" type="text" id="imageheight" name="imageheight" value="" size="4">
						</div>
					</div>
					<div class="form-group">
						<label class="control-label" for="imageborderwidth">Border Width</label>
						<input class="form-control numeric-field" type="text" id="imageborderwidth" name="imageborderwidth" value="0">
					</div>
					<div class="form-group">
						<div class="checkbox">
							<label>
								<input id="nonimagecontent" type="checkbox" name="nonimagecontent" value="1" onClick="dlgimg.inputImgBorderWidth(this)"> Non-image content
							</label>
						</div>
					</div>

				</div>

				<div class="form-group">
					<label class="control-label">Image Preview</label>
					<div id="disp" style="margin:0; width:100%; height:100px; overflow:auto;"></div>
				</div>
				<div class="form-group text-right">
					<button class="btn mce-btn mce-primary">Insert</button>
					<button class="btn btn-default active" onclick="dlgimg.close();">Cancel</button>
				</div>
			</form>
			</div>
		</div>
[[/include:advance_mode]]
		<div class="tab-pane" id="url">
			<div class="container" style="padding-left: 0px;">
				<br>
				<form id="url_frm" onSubmit="dlgimg.insertUrl();return false;" action="#" role="form" class="form-horizontal">
					<div class="form-group">
						<label class="control-label col-xs-2" for="img_url">URL</label>
						<div class="col-xs-10">
							<input class="form-control" id="img_url" name="img_url" value="">
						</div>
					</div>
					<div class="form-group">
						<label class="control-label col-xs-2" for="image_desc">Description</label>
						<div class="col-xs-10">
							<input class="form-control" type="text" id="url_image_desc" name="imagedesc" value="">
						</div>
					</div>
					<div class="form-group">
						<label class="col-xs-2 control-label">Dimension</label>
						<div class="col-xs-10">
							<div class="input-group input-append">
								<input class="form-control numeric-field" type="text" id="url_imagewidth" name="imagewidth" value="" size="4">
								x
								<input class="form-control numeric-field" type="text" id="url_imageheight" name="imageheight" value="" size="4">
							</div>
						</div>
					</div>
					<div class="form-group">
						<label class="control-label col-xs-2" for="imagealignment">Alignment</label>
						<div class="form-input col-xs-10">
							<select class="form-control" id="url_imagealignment" name="align"><option value="">None</option>[[insert_image_attribute]]</select>
						</div>
					</div>
					<div class="form-group">
						<label class="control-label col-xs-2" for="imageborderwidth">Border Width</label>
						<div class="col-xs-10">
							<input class="form-control numeric-field" type="text" id="url_imageborderwidth" name="imageborderwidth" value="0" size="2">
						</div>
					</div>
					<div class="form-group text-right">
						<button class="btn mce-btn mce-primary">Insert</button>
						<button class="btn btn-default active" onclick="dlgimg.close();">Cancel</button>
					</div>
				</form>
			</div>
		</div>
[[include:allow_file_upload]]
		<div class="tab-pane" id="upload">
			<div class="container">
			<form id="upload_frm" method="post" enctype="multipart/form-data" action="[[dlg_file]]" onSubmit="return dlgimg.insertFile();" role="form" class="form-horizontal">
			<input type="hidden" name="action" value="upload_file">
			<input type="hidden" name="section_id" value="[[section_id]]">
			<div class="form-group">
				<label class="control-label" for="library">Library</label>
				<select class="form-control" name="library" id="library"><option value="">== Select ==</option>[[upload_library_select]]</select>
			</div>
			<div class="form-group">
				<label class="control-label" for="fileupload">File</label>
				<input type="file" id="fileupload" name="fileupload">
			</div>
			<div class="form-group text-right">
				<button class="btn mce-btn mce-primary">Upload</button>
				<button class="btn btn-default active" onclick="dlgimg.close();">Cancel</button>
			</div>
			</form>
			</div>
		</div>
	</div>
[[/include:allow_file_upload]]

</div>
</div>

</body>
</html>
END
	return ExSite::Misc::substitute($template,\%tdata);
}

#########################################
package ExSite::Dialog::Link;
use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Section;

use ExSite::Dialog;
use base qw(ExSite::Dialog::Base);

sub run{
	my ($this,$INPUT) = @_;
	my $input = $INPUT->combine();
	my $ml = $this->ML();
	my $dlg_mode = $input->{mode};
	if ($input->{action} eq "getimagelist"){
		return $this->getImageList($input);
	} elsif ($input->{action} eq "getlib"){
		return $this->getlib($input->{section_id},$input->{name});
	} elsif ($input->{action} eq "geturl"){
		return $this->geturl($input->{section_id},$input->{url});
	} elsif ($input->{action} eq "getcontentname"){
		return $this->getcontentname($input->{section_id},$input->{url});
	}
	return $this->content($ml->html($ml->head($ml->title("Insert/Edit Link")).$ml->body($this->form($dlg_mode,$input,$this->fetch_page($input)))));
}
sub getcontentname{
	my ($this,$sectionid,$url) = @_;
	my $newurl = $url;
	if ($newurl =~ /^.*?name=(.+?)(&(.*?))?$/){
		$newurl =~ s/^.*?name=(.+?)(&(.*?))?$/$1/gis;
	}
	if ($newurl =~ /\[\[.+?\]\]/){
		$newurl =~ s/\[\[(.+?)\]\]/$1/gis;
	}
	return $this->content($newurl);
}
sub geturl{
	my ($this,$sectionid,$url) = @_;
	my $newurl;
	{
		$url =~ /^.*?href=(\"|')?([^>]+?)(\s|\"|').*$/gis;
		$newurl = $2;
	}
	return $this->content($newurl);
}
sub getlib{
	my ($this,$sectionid,$name) = @_;
	my $libid = 0;
	if ($sectionid && $name){
		my $s = ExSite::Section->new(id=>$sectionid);
		$s->prefetch();
		my $c = $s->find($name);
		$libid = $c->get("content")->{page_id};
	}
	return $this->content($libid);
}
# Fetch pages
sub fetch_page{
	my ($this,$query) = @_;
	if (!$query->{section_id}){
		return [];
	}
	# Fetch pages
	my @pages = $this->DB()->fetch_match("page",{section_id=>$query->{section_id},type=>"page"},"title");
	return \@pages;
}
# create a library drop down list
sub getLibrarySelector{
	my ($this,$in) = @_;
	my $ml = $this->ML();
	my $len = 50;
	my $libs =$this->fetch_library($in->{section_id});
	my @options = map {$ml->option($this->truncate_text($_->{filename},$len),{value=>"$_->{page_id}"})} @$libs;
	return $ml->select($ml->option("== Select Library ==",{value=>""}).join("",@options),{id=>"lib_id",name=>"lib_id",onChange=>"dlglnk.getImageList()",class=>"form-control"});
}
# Image/document drop down list
sub getImageList{
	my ($this,$in) = @_;
	my $ml = $this->ML();
	my %page = $this->DB->fetch("page",$in->{page_id});
	my @content_list = $this->DB()->fetch_match("content",{page_id=>$in->{page_id}});
	my @options = map {
		my $val = "[[$page{filename}/$_->{name}]]";
		$ml->option($this->truncate_text($_->{name}),{value=>$val})
	} @content_list;
	return "content-type: text/html\n\n".$ml->select($ml->option("== Select Image/Document ==",{value=>""}).join("",@options),{name=>"obj_id",onChange=>"dlglnk.selectLibrary()",class=>"form-control"});
}
sub script{
	return "$config{server}{server}$config{server}{CGIpath}/$config{form}{tinymce}{dialog}{dlglink}{script}";
}

sub form{
	my ($this,$mode,$in,$links) = @_;
	my $ml = $this->ML();
	my %template_data = (
		tinymce_path=>"$config{server}{server}$config{server}{HTMLpath}$config{form}{tinymce}{path}",
		jquery_js=>$config{jquery},
		url=>($in->{url} || ""),
	);
	if ($mode == 2){
		$template_data{basic_mode} = 1;
	}else{
		my $len = 47;
		my @links = map {{id=>$_->{page_id},url=>"$_->{filename}",text=>(length($_->{label}) > $len ? substr($_->{label},0,$len)."..."  : $_->{label})." ($_->{filename})"}} sort {uc($a->{label}) cmp uc($b->{label})} @$links;
		my $section_id = $in->{section_id} || 0;
		$template_data{select_library} = $this->getLibrarySelector($in);
		$template_data{httprequest_js} = "$config{server}{server}$config{server}{HTMLpath}/_ExSite/js/httprequest.js";
		$template_data{advance_mode} = 1;
		$template_data{page_list} = scalar(@$links) > 0 ? "var pageref = new Array();".join("",map {"pageref[\"$_->{filename}\"] = $_->{page_id};"} @$links) : "";
		$template_data{enable_cms_cmd} = ($section_id ? 1 : 0);
		$template_data{dlg_file} = $this->script();
		$template_data{section_id} = ($section_id || 0);
		# Pages:
		$template_data{select_internal_page} .= $ml->option("==".(scalar @links > 0 ? "Select Internal Page" : "No Pages Found")."==",{value=>""});
		map { $template_data{select_internal_page} .= $ml->option($this->truncate_text($_->{text},$len),{value=>"{{$_->{url}}}"}) } @links;
	}
	my $email_re = '[\w][\w\-\+\.]*@[\w\-\+\.]+\.[A-Za-z]{2,30}';
	my $template = <<END;
<!DOCTYPE html>
<html lang="en">
<head>
<meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no">
</head>
<body>

<script src="[[jquery_js]]"></script>

<!-- Latest compiled and minified CSS -->
<link rel="stylesheet" href="//netdna.bootstrapcdn.com/bootstrap/3.0.2/css/bootstrap.min.css">

<!-- Optional theme -->
<link rel="stylesheet" href="//netdna.bootstrapcdn.com/bootstrap/3.0.2/css/bootstrap-theme.min.css">

<!-- Latest compiled and minified JavaScript -->
<script src="//netdna.bootstrapcdn.com/bootstrap/3.0.2/js/bootstrap.min.js"></script>


[[include:advance_mode]]<script src="[[httprequest_js]]"></script>[[/include:advance_mode]]

<link rel="stylesheet" href="$config{server}{server}$config{server}{HTMLpath}$config{form}{tinymce}{path}/skins/lightgray/skin.min.css">

<script>
[[include:advance_mode]]
[[page_list]]
[[/include:advance_mode]]
var dlglnk = {
	closeFlag : 0,
	close : function(){
		this.closeFlag = 1;
	},
	setAttrib : function (elm, attrib, value) {
		var valueElm = document.forms.dialog_form.elements[attrib.toLowerCase()];
		var dom = parent.tinymce.activeEditor.dom;
		if (typeof(value) == "undefined" || value == null) {
			value = "";
			if (valueElm)
				value = valueElm.value;
		}
		// Clean up the style
		if (attrib == 'style') value = dom.serializeStyle(dom.parseStyle(value), 'a');
		dom.setAttrib(elm, attrib, value);
	},
	setAllAttribs : function (elm) {
		var href = new String(document.forms.dialog_form.linkURL.value);
[[include:advance_mode]]
		if (jQuery("input[name='link_to']:checked").val() == 'url'){
[[/include:advance_mode]]
			if (!href.match(/^#/) && !href.match(/^mailto:/i)){
				if (!href.match(/^http/i) && !href.match(/^\\//)){
					href = 'http://' + href;
				}
			}
[[include:advance_mode]]
		}
[[/include:advance_mode]]
		var target = jQuery("input[type='checkbox'][name='newwindow']").is(":checked") == true ? '_blank' : "";
		this.setAttrib(elm, 'href', href);
		this.setAttrib(elm, 'target', target == '_self' ? '' : target);
		// Refresh in old MSIE
		if (parent.tinymce.isMSIE5)
			elm.outerHTML = elm.outerHTML;
	},
	insertAction : function () {
		if (this.closeFlag){
			(parent.tinymce.activeEditor || top.tinymce.activeEditor).windowManager.close();
			return;
		}
		var inst = parent.tinymce.activeEditor;
		var elm, elementArray, i, url, txt;
		elm = inst.dom.getParent(inst.selection.getNode(), 'a[href]');
		url = document.forms.dialog_form.linkURL.value;
		// Remove element if there is no href
		if (!url) {
			inst.execCommand("unlink");
			(parent.tinymce.activeEditor || top.tinymce.activeEditor).windowManager.close();
			return;
		}
		inst.execCommand("mceBeginUndoLevel");
		// Create new anchor elements
		if (elm == null) {
[[include:advance_mode]]
			if (\$("input[name='link_to']:checked").val() == 'url'){
[[/include:advance_mode]]
			if (/${email_re}/.test(url) && !/^mailto:/.test(url)){
				url = 'mailto:' + url;
			}
			if (!url.match(/^#/) && !url.match(/^mailto:/i)){
				if (!url.match(/^http/i) && !url.match(/^\\//)){
					url = 'http://' + url;
				}
			}
[[include:advance_mode]]
			}
[[/include:advance_mode]]
			var is_aElm = /^<a [^>]+>[^<]+<\\/a>\$/.test(inst.selection.getContent());
			var linkattrs = {
				'href' : url
			};
			if(jQuery("input[type='checkbox'][name='newwindow']").is(":checked") == true){
				linkattrs.target = '_blank';
			}
			if (this.getClass()){
				linkattrs.class = this.getClass();
			}
			if (!is_aElm){
				inst.insertContent(inst.dom.createHTML('a', linkattrs, inst.selection.getContent()));
			} else {
				inst.execCommand('mceInsertLink', false, linkattrs);
			}
		} else {
			this.setAllAttribs(elm);

			this.setAttrib(elm, 'class', this.getClass());

			inst.selection.select(elm);
		}
		// Don't move caret if selection was image
		inst.focus();
		inst.selection.select(elm);
		(parent.tinymce.activeEditor || top.tinymce.activeEditor).windowManager.close();
	},
	getClass : function(){
		var _link_class = '';
[[include:advance_mode]]
		if (jQuery("input[type='checkbox'][name='make_button']").is(":checked") == true){
			_link_class = "button";
		}
[[/include:advance_mode]]
		var _class = jQuery("input[type='hidden'][name='link_class']").val();
		_class = typeof(_class) == 'undefined' ? "" : _class;
		if (_class){
			_link_class = _link_class ? _link_class + ' ' + _class : _class;
		}
		return _link_class;
	},
[[include:advance_mode]]
	getUrl : function(){
		switch(this.link_type){
		case 'imgdoc':
			return this.default_imgdoc;
			break;
		case 'page':
			return this.default_page;
			break;
		default:
			return this.default_url
			break;
		}
	},
	isImgDocObject : 0,
	default_url : '',
	default_page : '',
	default_imgdoc : '',
	default_lib : 0,
	link_type : '',
	enable_cms_op : [[enable_cms_cmd]],
	getImageList : function (){
		var list = getRequestData('[[dlg_file]]?action=getimagelist&page_id=' + document.forms.dialog_form.lib_id.value);
		document.getElementById('imageSelector').innerHTML = list;
	},
	selectMade : function () {
		var choice = document.forms.dialog_form.linkSelect.options[document.forms.dialog_form.linkSelect.selectedIndex];
		if (choice.value) {
			document.forms.dialog_form.linkURL.value = choice.value;
		}
	},
	selectLibrary : function (){
		this.isImgDocObject = 1;
		var choice = document.forms.dialog_form.obj_id.options[document.forms.dialog_form.obj_id.selectedIndex];
		if (choice.value){
			document.forms.dialog_form.linkURL.value = choice.value;
		}
	},
	resetType : function (){
		this.isImgDocObject = 0;
	},
	linktoUpdate : function (val){
		this.link_type = val || 'url';
		if (!this.enable_cms_op){
			this.link_type = 'url';
		}
		this.select_linkto_radio(this.link_type);
		switch(this.link_type){
		case 'imgdoc':
			document.getElementById('div_url').style.display = 'none';
			document.getElementById('div_internal_page').style.display = 'none';
			document.getElementById('div_image_document').style.display = 'block';
			document.forms.dialog_form.link_url.value = '';
			if (this.default_lib){
				for(var i = 0; i < document.forms.dialog_form.lib_id.length; i++){
					if (document.forms.dialog_form.lib_id.options[i].value == this.default_lib){
						document.forms.dialog_form.lib_id.selectedIndex = i;
						break;
					}
				}
			}
			this.getImageList();
			if (this.default_imgdoc){
				var selected_val = '\\[\\[' + this.default_imgdoc + '\\]\\]';
				for(var i = 0; i < document.forms.dialog_form.obj_id.length; i++){
					if (document.forms.dialog_form.obj_id.options[i].value == selected_val){
						document.forms.dialog_form.obj_id.selectedIndex = i;
						document.forms.dialog_form.linkURL.value = selected_val;
						break;
					}
				}
			}
			this.link_type = 'imgdoc';
			break;
		case 'page':
			document.getElementById('div_url').style.display = 'none';
			document.getElementById('div_image_document').style.display = 'none';
			document.getElementById('div_internal_page').style.display = 'block';
			document.forms.dialog_form.link_url.value = '';
			if (this.default_page){
				var selected_val = '{{' + this.default_page + '}}';
				for(var i = 0; i < document.forms.dialog_form.linkSelect.options.length; i++){
					if (document.forms.dialog_form.linkSelect.options[i].value == selected_val){
						document.forms.dialog_form.linkSelect.selectedIndex = i;
						document.forms.dialog_form.linkURL.value = selected_val;
						break;
					}
				}
			}
			this.link_type = 'page';
			break;
		default:
			document.getElementById('div_image_document').style.display = 'none';
			document.getElementById('div_internal_page').style.display = 'none';
			document.getElementById('div_url').style.display = 'block';
			document.forms.dialog_form.link_url.value = this.default_url || '';
			this.link_type = 'url';
			break;
		}
	},
	select_linkto_radio : function (radio_val){
		for(var i = 0; i < document.forms.dialog_form.link_to.length; i++){
			if (document.forms.dialog_form.link_to[i].value == radio_val){
				document.forms.dialog_form.link_to[i].checked = true;
			}else{
				if (!this.enable_cms_op){
					document.forms.dialog_form.link_to[i].disabled = true;
				}
				document.forms.dialog_form.link_to[i].checked = false;
			}
		}
		if (!this.enable_cms_op){
			var inactive_color = '#666666';
			document.getElementById("label_link_page").style.color = inactive_color;
			document.getElementById("label_link_imgdoc").style.color = inactive_color;
		}
	},
[[/include:advance_mode]]
	init : function (){
		this.closeFlag = 0;
		var inst = parent.tinymce.activeEditor,elm,action,link_url;
		elm = inst.selection.getNode();
		a_elm = inst.dom.getParent(elm, 'a[href]');
		action = 'insert';
		var href = inst.dom.getAttrib(a_elm,'href');
		var target = inst.dom.getAttrib(a_elm,'target');
		if (!href){
			if (/<a /.test(inst.selection.getContent())){
				var hmatch = inst.selection.getContent().match(/href=("|')([^"]*")/g);
				if (hmatch){
					hmatch = hmatch[0];
					href = hmatch.replace(/^href=("|')/i,'');
					href = href.replace(/("|')\$/,'');
				}
				var tmatch = inst.selection.getContent().match(/target=("|')([^"]*")/g);
				if (tmatch){
					tmatch = tmatch[0];
					target = tmatch.replace(/^target=("|')/i,'');
					target = target.replace(/("|')\$/,'');
				}
			}
		}
		if ((elm != null && elm.nodeName == 'A') || href){
			action = "update";
			if ((target).match(/_blank/i)){
				document.forms.dialog_form.newwindow.checked = true;
			}

			var l_class = inst.dom.getAttrib(a_elm,'class') || '';
			if (l_class){
				var _class = new String(l_class);
				var class_list = _class.split(' ');
				var class_hidden = new Array();
				for(var i = 0; i < class_list.length;i++){
					if (class_list[i] == 'button'){
						document.forms.dialog_form.make_button.checked = true;
					}else{
						class_hidden.push(class_list[i]);
					}
				}
				document.forms.dialog_form.link_class.value = class_hidden.join(" ");
			}

		}
		link_url = unescape(href);
		if (action == "update"){
[[include:advance_mode]]
			if (link_url.match(/content.cgi/i) || link_url.match(/^\\[(\\|)?\\[.+?\\](\\|)?\\]/)){
				if (link_url.match(/^\\[(\\|)?\\[.+?\\](\\|)?\\]/)){
					link_url.replace(eval(/\\|/g),'');
				}
				var ajax_imgdoc_url = '[[dlg_file]]?action=getcontentname&section_id=[[section_id]]&url=' + escape(link_url);
				var imgdoc_name = getRequestData(ajax_imgdoc_url);
			this.default_imgdoc = imgdoc_name;
				var fetch_lib_url = '[[dlg_file]]?action=getlib&section_id=[[section_id]]&name=' + this.default_imgdoc;
				this.default_lib = getRequestData(fetch_lib_url);
				this.linktoUpdate('imgdoc');
			}else if (link_url.match(/^\\{(\\|)?\\{.+?\\}(\\|)?\\}/)){
				var cms_page = link_url;
				cms_page = cms_page.replace(eval(/\\|/g),'');
				cms_page = cms_page.replace(eval('/^{{(.+?)}}\$/'),"\$1");
				var page_name;
				if (cms_page.match(/^\\d+\$/)){
					for (var x in pageref){
						if (pageref[x] == cms_page){
							page_name = x;
							break;
						}
					}
				}else{
					page_name = cms_page;
				}
				if (page_name){
					this.default_page = page_name;
					this.linktoUpdate('page');
				}else{
					this.linktoUpdate('url');
				}
			}else{
				this.default_url = link_url;
				this.linktoUpdate('url');
			}
[[/include:advance_mode]]
[[include:basic_mode]]
			document.forms.dialog_form.link_url.value = link_url;
[[/include:basic_mode]]
		}
	}
};
jQuery(document).ready(function(){
dlglnk.init();
});
</script>

<body>

<form id="dialog_form" onsubmit="dlglnk.insertAction();return false;" action="#">
<div class="container">
[[include:advance_mode]]
	<div class="form-group">
		<label>Type</label>
		<div class="radio">
			<label>
			<input type="radio" value="url" name="link_to" onclick="dlglnk.linktoUpdate('url')" checked><span id="label_link_uri">URL</span>
			</label>
		</div>
		<div class="radio">
			<label>
			<input type="radio" value="page" name="link_to" onclick="dlglnk.linktoUpdate('page')"><span id="label_link_page">Internal Page</span>
			</label>
		</div>
		<div class="radio">
			<label>
			<input type="radio" value="imgdoc" name="link_to" onclick="dlglnk.linktoUpdate('imgdoc')"><span id="label_link_imgdoc">Image/Document</span>
			</label>
		</div>
	</div>
[[/include:advance_mode]]

	<div class="form-group">
		<label>To</label>
[[include:advance_mode]]
		<div class="form-inline">

		<div id="div_internal_page" style="display: none;">
			<select class="form-control" name="linkSelect" onChange="dlglnk.selectMade()">[[select_internal_page]]</select>
		</div>
		<div id="div_image_document" style="display:none;">
			[[select_library]]
			<div id="imageSelector">
				<select class="form-control" name="obj_id" onChange="dlglnk.showimage()"><option value="">== Select Image/Document ==</option></select>
			</div>
		</div>

		</div>
[[/include:advance_mode]]
		<div id="div_url" style="display: block;">
			<input class="form-control" type="text" id="link_url" name="linkURL" value="[[url]]" size="50"[[include:advance_mode]] onChange="dlglnk.resetType()"[[/include:advance_mode]]>
		</div>
		<div class="checkbox">
			<input type="checkbox" name="newwindow"> Open this link in a new window
[[include:advance_mode]]
			<br>
			<input type="checkbox" name="make_button"> Add button style
[[/include:advance_mode]]
			<input type="hidden" name="link_class" value="">
		</div>
	</div>
	<div class="form-group text-right">
		<button class="btn mce-btn mce-primary">Insert</button>
		<button class="btn btn-default active" onClick="dlglnk.close();">Cancel</button>
	</div>
</div>
</form>

</body>
</html>
END
	return ExSite::Misc::substitute($template,\%template_data);
}
1;
