#!/usr/bin/perl
#-----------------------------------------------------------------------
# captcha.cgi - generate captcha information
#
# Usage:
#
#    captcha.cgi?XXXXXXX...     generate an image of the encoded string
#
#    captcha.cgi?key=YYY        generate a complete captcha
#
#    captcha.cgi?postkey=YYY    generate a passive captcha (posting key)
#
# In the latter 2 cases, we can also add a version=lang parameter to pass
# our version/language.
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Crypt;
use ExSite::Input;
use ExSite::Captcha;

if ($ENV{HTTP_USER_AGENT} =~ /msnbot|Googlebot|Yahoo/i) {
    # looks like a robot - don't even bother
    print "content-type:text/plain\n\n";
    print "Silly robot.";
    exit;
}

# abbreviated setup - so that captchas don't mess with cookies or sessions
# accidentally.
&ExSite::Config::exsite_init_store;
&ExSite::Config::exsite_init_config;
&ExSite::Config::exsite_init_msg;

my $in = new ExSite::Input;
my $q = $in->query();

if ($q->{version}) {
    if (! $share{DB}) {
	# database is not setup by default, but we need it to get translations
	require ExSite::DB;
	$share{DB} = new ExSite::DB();
    }
    (tied %msg)->version($q->{version});
}

if ($q->{postkey}) {
    # AJAX-generated passive captcha
    &make_post_key();
}
elsif (exists $q->{key}) {
    # AJAX-generated interactive (image) captcha
    &make_captcha();
}
else {
    # generate the captcha image itself
    &make_captcha_image();
}

# generate a post key for a passive captcha

sub make_post_key {
    my $c = new ExSite::Captcha;
    my $postkey = $c->get_post_key($q->{postkey});
    print "Content-type: text/plain\n\n";
    print "$postkey";
}

# generate a complete image captcha

sub make_captcha {
    # disallow text mode switing in AJAX mode
    my $c = new ExSite::Captcha(mode=>$q->{captchamode});
    my $captcha = $c->make($q->{key});
    print "content-type: text/html\n\n";
    print $captcha;
}

# generate the captcha image

sub make_captcha_image {
    if ($config{token}{captcha} && $config{token}{type} eq "db") {
	# need DB access
	require ExSite::DB;
	my $db = new ExSite::DB;
    }
    my $c = new ExSite::Captcha;
    my $imgdata = $c->make_captcha_image($ENV{QUERY_STRING});
    print "content-type:image/png\n\n";
    print $imgdata;
}
