#!/usr/bin/perl
#!/usr/bin/speedy
#-----------------------------------------------------------------------
# ctrl-panel.cgi - generate a control panel for a web app
#
# Usage:  /cgi-bin/ctrl-panel.cgi/WEBAPP?options
#
# WEBAPP is the name of an application installed in the Modules area.
#
# All options are assumed to be directed at the Webapp, except:
# 
# _bare=1    Do not show the usual ExSite ctrlpanel wrapper.
#
#-----------------------------------------------------------------------
#
#   Copyright 2001-2011 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Form;  # use Form if inputting or processing forms
use ExSite::Module qw(get_module icon);
use ExSite::Input;
use ExSite::Misc;
use ExSite::Util;
use ExSite::ML;
#use ExSite::HTML;

use vars qw($db $query $ml $in $icon $name $info $title $out
	    $path @path $module $BARE $npane
	    $utime1 $stime1 $cutime1 $cstime1);
($utime1,$stime1,$cutime1,$cstime1) = times;
&exsite_init;
$db = new ExSite::Form;
$ml = &get_obj("ML");

# process our inputs

$in = new ExSite::Input;
$query = $in->query();           # get query data
$BARE = $query->{_bare};         # BARE is usually true for popups and dialogs
$path = $in->path_info();        # get path info
@path = split/\//,$path;
$module = $path[1];              # 1st element of path is the Module
$in->path("module","/$module");  # define module path segment in case there is 
                                 # more path data

# setup JS
my $js = &js();

# get our content

if ($db->adjust_level > 1) {
    my $app = &get_module($module);
    if ($app) {
	$icon = &ExSite::Module::icon($module,$config{iconset}); #$app->icon;
	$name = $app->ioctl('ModuleName') || $module;
	$title = "$name - $config{site}{name}";
	my $panel = $app->ioctl('ControlPanel');
	if (ref $panel eq "CODE") {
	    $out .= &ctrl_panel_1_pane($app,$panel);
	    $npane = 1;
	}
	elsif (ref $panel eq "ARRAY" && $BARE) {
	    # only fetch the main panel in BARE mode
	    $out .= &ctrl_panel_1_pane($app,$panel->[1]);
	    $npane = 1;
	}
	elsif ((ref $panel) eq "ARRAY") {
	    $out .= &ctrl_panel_2_pane($app,$panel);
	    $npane = 2;
	}
	else {
	    $out .= $ml->div($ml->h1($msg{"Error executing $module control panel"}),{class=>"ctrlpanel"});
	    $npane = 1;
	}
    }
    else {
	$icon = "$config{server}{HTMLpath}/_ExSite/images/exsite-button-round.gif";
	$title = $msg{"Error"};
	if ($@) {
	    if ($module =~ /\[\[(.+)\]\]/) {
		# bad CMS link?
		$out .= $ml->h1("This link works on regular web pages only").
		$ml->p("The link to ".$ml->strong($1)." cannot be resolved from an administrator control panel. To see where this link goes, you must view this content from a regular web page.");
	    }
	    else {
		$out .= $ml->h1(&substitute("Failed to start module [[module]]",
					    {module=>$module})).$ml->p(join $ml->br, split(/\n/,$@));
	    }
	}
	else {
	    $out .= $ml->h1(&substitute("[[module]]: Permission denied (no license)",{module=>$module}));
	}
	$out = $ml->div($out,{class=>"ctrlpanel"});
	$npane = 1;
    }
}
else {
    $out .= $ml->h1($msg{"Permission denied."}).
	$ml->p($msg{"You do not have permission to access this control panel."});
    $out .= $share{DB}->login_form(undef,$config{server}{server}.&relink());
    $out = $ml->div($out,{class=>"ctrlpanel"});
    $npane = 1;
}

# initialize our page
$ml->Doctype($config{doctype});
$ml->Doc($out);

# wrap the content in some boilerplate HTML
if ($BARE) {
    # add some controls to the top
    if ($BARE == 1) {
	$ml->Prepend(
	    $ml->div(
		$ml->a("&times;&nbsp;&nbsp;close",{href=>"javascript:window.close()"}),
		{class=>"close_button"} 
	    )
	    );
	# wrap with a body
	$ml->__body({class=>"ctrlpanel"});
	# add a head
	$ml->Prepend(
	    $ml->head(
		$ml->title($title).
		$ml->link(undef,
			  {rel=>"stylesheet",
			   type=>"text/css",
			   href=>"$config{server}{HTMLpath}/_ExSite/css/ExSite.css"})
	    )
	    );
    }
}
else {
    my $topbar = &topbar($title,$icon,$name,$module);
    $ml->Prepend($topbar);
    if ($npane == 1) {
	$ml->__div({class=>"ctrlpanel_1pane"});
    }
    $ml->__body({class=>"ctrlpanel",
		 marginheight=>0,marginwidth=>0,topmargin=>0,leftmargin=>0,
		 rightmargin=>0});
}

# CSS

if ($BARE != 2) {
    my $css = &css();

    # add a head
    $ml->Prepend(
	$ml->head(
	    $ml->meta(undef,
		      {"http-equiv"=>"Content-Type",
		       content=>"text/html; charset=$config{charset}"}).
	    $ml->title($title).
	    $ml->link(undef,{rel=>"shortcut icon",href=>"$config{server}{HTMLpath}/_ExSite/images/exsite.ico"}).
	    $css.
	    $js
	)
	);
    $ml->__html();
}
# adapted from $ml->PrintWithHeader();
if (not $ml->{canceled}) {
    print "X-XSS-Protection: 0\n";
    print "Content-type: text/html\n\n";
    $ml->Print();
}

&exsite_close();

### END

sub ctrl_panel_1_pane {
    my ($app, $panel) = @_;
    my $pane = &ctrl_panel_pane($app,$panel);
    $pane .= $ml->div(&debug,{class=>"ctrlpanel_debug"});
    return $ml->div($pane,{class=>"ctrlpanel"});
}

sub ctrl_panel_2_pane {
    my ($app, $panel) = @_;
    return 
	$ml->table(
	    $ml->tr($ml->td(undef,{class=>"ctrlpanel_2pane_head",colspan=>2})).
	    $ml->tr(
		$ml->td(
		    &ctrl_panel_pane($app,$panel->[0]),
		    {class=>"ctrlpanel_sidebar"}
		).
		$ml->td(
		    &ctrl_panel_pane($app,$panel->[1]).
		    $ml->div(&debug,{class=>"ctrlpanel_debug"}),
		    {class=>"ctrlpanel"}
		)
	    ),
	    {class=>"ctrlpanel_2pane"}
	);
}

sub ctrl_panel_2_pane_notable {
    my ($app, $panel) = @_;
    return 
	$ml->div(
	    $ml->div(undef,{class=>"ctrlpanel_sidebar_head"}).
	    &ctrl_panel_pane($app,$panel->[0]),
	    {class=>"ctrlpanel_sidebar"}
	).
	$ml->div(
	    &ctrl_panel_pane($app,$panel->[1]).
	    $ml->div(&debug,{class=>"ctrlpanel_debug"}),
	    {class=>"ctrlpanel"}
	);
}

sub ctrl_panel_pane {
    my ($app, $subpanel) = @_;
    my $pane;
    if (ref $subpanel eq "CODE") {
	$pane = ( eval { $app->$subpanel } || 
		 join $ml->br,(split /\n/,$@) );
    }
    elsif (ref $subpanel eq "ARRAY") {
	$pane = $share{DB}->error("Nested subpanel types not supported yet");
    }
    elsif ($subpanel eq "section") {
	$pane = &section_list();
    }
    else {
	$pane = $db->error("Unknown subpanel type: $subpanel");
    }
    return $pane;
}

sub topbar {
    my ($title,$icon,$name,$module) = @_;
    my $help = (-e "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/$module/help.html") ?
	$ml->td(
		$ml->a(
		       $ml->img(undef,{src=>"$config{server}{HTMLpath}/_ExSite/images/button-help.gif",height=>30,width=>30,border=>0,alt=>"Help",title=>"Help"}),
		       { href=>"javascript:popup_medium('$config{server}{HTMLpath}/_Modules/$module/help.html')" }
		       ),
		) : "";

    return $ml->div(
	$ml->table(
	    $ml->tr(
		$ml->td(
		    $ml->img(undef,{src=>$icon,height=>32,width=>32,border=>0,alt=>$name})
		).
		$ml->td(
		    $ml->span($title,{class=>"title"}),
		    {class=>"titlebar"}
		).
		$ml->td(
		    $ml->a(
			$ml->img(undef,{src=>"$config{server}{HTMLpath}/_ExSite/images/button-home.gif",height=>30,width=>30,border=>0,alt=>"Home",title=>"Home"}),
			{ href=>"$config{server}{server}$config{server}{CGIpath}/$config{prog}{ctrlpanel}/$module" }
		    ),
		).
		$help,
	    ),
	    {class=>"titlebar"}
	    ),
	{class=>"topbar"}
	);
}

sub debug {
    if ($config{debug}) {
	return &ExSite::HTML::HelpPopup(
					label=>"Debug Info",
					message=>&debug_status($utime1,$stime1,$cutime1,$cstime1),
					width=>750,
					);
    }
    return undef;
}

sub section_list {
    my (@nesttype) = @_;
    my $in = new ExSite::Input;
    my $q = $in->query;
    
    my $current_site = $share{DB}->this_site();
    my @data = $share{DB}->my_real_sites();
    my (%section,%page);
    my $list;

    foreach my $s (@data) {
	my $sattr = {href=>&relink(section_id=>$s->{section_id})};
	my $lattr;
	if ($s->{section_id} == $current_site->{section_id}) {
	    $sattr->{class} = "this_section";
	    $lattr = { class=>"open" };
	}
	my $item = $ml->a($s->{title},$sattr);
	$list .= $ml->li($item,$lattr);
    }
    return $ml->ul($list,{id=>"browser",class=>"treeview-gray"});
}

sub js {
    my $js = $ml->script(undef,
			 {type=>'text/javascript',
			  src=>"$config{server}{HTMLpath}/_ExSite/js/misc.js"});
    if ($config{jquery} && ! $share{js}{jquery}) {
	# add standard jquery link
	$js .= $ml->script(undef,{type=>'text/javascript',src=>$config{jquery}});
	$share{js}{jquery} = 1;
	if ($config{jqueryui} && ! $share{js}{jqueryui}) {
	    $js .= $ml->script(undef,{type=>'text/javascript',src=>$config{jqueryui}});
	    $share{js}{jqueryui} = 1;
	}
	### what uses this??
	$js .= $ml->script(undef,{type=>'text/javascript',src=>"$config{server}{HTMLpath}/_ExSite/js/jquery.cookie.js"});
	if ($npane == 2) {
	    # include treeview plugin for sidebar nav
	    $js .= $ml->script(undef,{type=>'text/javascript',src=>"$config{server}{HTMLpath}/_ExSite/js/jquery.treeview.js"});
	    $js .= $ml->script('
$(document).ready(function(){
	$("#browser").treeview({
                unique: true,
                collapsed: true
	})
});', {type=>'text/javascript'});
	}
    }
    if (-e "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/$module/$module.js") {
	# add a custom JS file
	$js .= $ml->script(undef,
		     {type=>'text/javascript',
		      src=>"$config{server}{HTMLpath}/_Modules/$module/$module.js"});
    };
    return $js;
}

sub css {
    my $css = $ml->link(undef,
			{rel=>"stylesheet",
			 type=>"text/css",
			 href=>"$config{server}{HTMLpath}/_ExSite/css/ExSite.css"});
    if (-e "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/$module/$module.css") {
	# add a custom stylesheet
	$css .= $ml->link(undef,
			  {rel=>"stylesheet",
			   type=>"text/css",
			   href=>"$config{server}{HTMLpath}/_Modules/$module/$module.css"});
    }
    return $css;
}
