#!/usr/bin/perl
#!/usr/bin/speedy
#-----------------------------------------------------------------------
# home.cgi - ExSite WebTop
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::PageBuf;
use ExSite::Misc;
use ExSite::HTML;
use ExSite::Input;
use ExSite::ML;
use ExSite::Module qw(module_list get_module icon);
use ExSite::Report;

use vars qw($db %query $popup_flag $input $in $section_id);

&exsite_init;
# connect to database
$db = new ExSite::DB;
$in = new ExSite::Input;
$input = $in->query;
$section_id = $input->{section_id};
if (! defined $section_id && $config{webtop}{section_selector} =~ /^\d+$/) {
    $section_id = $config{webtop}{section_selector};
}
my $ml = new ExSite::ML;
$ml->Doctype($config{doctype});

#my $access = $db->authorize;
my $access = $db->adjust_level;
my $webtop_access = &webtop_access;
my $webtop = 0;
my %icongroup;

if ($webtop_access == 1) {

    # Web Top

    my @modules = ($access==2 && $config{webtop}{whitelist}) ? 
	@{$config{webtop}{whitelist}} :
	&module_list;
	
    my %category = map { $_=>1 } @{$config{webtop}{categories}};
    my %favourites;
    my $faves = &preference("webtop.favourites");
    if ($faves) {
	%favourites = map { $_ => 1 } split /,\s*/, $faves;
    }
    my $errors;
    my %moduleinfo;
    foreach my $modname (@modules) {
	my $mod = &get_module($modname,undef,$section_id);
	if ($mod) {
	    if (tied %session) {
		$moduleinfo{$modname} = {
		    ModuleName => $mod->ioctl("ModuleName"),
		    ControlPanel => defined $mod->ioctl("ControlPanel"),
		    Publish => defined $mod->ioctl("Publish"),
		    Unpublish => defined $mod->ioctl("Unpublish"),
		    AdminOnly => $mod->ioctl("AdminOnly"),
		};
	    }
	    my $mod_access = $mod->ioctl('Access');
	    if (! defined $mod_access || $mod_access <= $access) {
		my $panel = $mod->ioctl('ControlPanel');
		if ($panel) {
		    # skip this module if it has been renamed to blank
		    next if (exists $config{webtop}{module_name}{$modname} && 
			     ! $config{webtop}{module_name}{$modname});
		    # a control panel exists for this module;
		    my $iconurl = &icon($modname,$config{iconset});
		    # find the control panel URL
		    my $url;
		    if ((ref $panel) =~ /CODE|ARRAY/) {
			$url = "$config{server}{CGIpath}/$config{prog}{ctrlpanel}/$modname";
		    }
		    else {
			# assume ioctl has returned the control panel url
			$url = $panel;
		    }
		    if ($section_id) {
			$url .= "?section_id=$section_id";
		    }
		    # make an icon
		    my $icon = $ml->img(undef,{src=>$iconurl, height=>64, width=>64, border=>0, alt=>""});
		    $icon .= $ml->br();
		    my $icon_label = 
			$config{webtop}{module_name}{$modname} || 
			$mod->ioctl('ModuleName') || 
			$modname;
		    $icon .= $msg{$icon_label};
		    # wrap icon in a link
		    my $link;
		    if ($config{webtop}{popup_style} eq "tab") {
			$link = $ml->a($icon,{class=>"webtop",href=>$url,target=>"_blank"});
		    }
		    else {
			my $size = $mod->ioctl("Size") || "large";
			if (ref $size eq "ARRAY") {
			    $link = $ml->a($icon,{class=>"webtop",href=>"javascript:popup_custom($size->[0],$size->[1],'$url')"});
			}
			elsif ($size eq "large") {
			    $link = $ml->a($icon,{class=>"webtop",href=>"javascript:popup_large('$url')"});
			}
			elsif ($size eq "medium") {
			    $link = $ml->a($icon,{class=>"webtop",href=>"javascript:popup_medium('$url')"});
			}
			elsif ($size eq "small") {
			    $link = $ml->a($icon,{class=>"webtop",href=>"javascript:popup_small('$url')"});
			}
		    }
		    my $category = $mod->ioctl('Category') || "Applications";
		    if (ref $category eq "ARRAY") {
			# multiple categories; use first one that already exists
			foreach my $cat (@$category) {
			    if (exists $category{$cat}) {
				$category = $cat;
				last;
			    }
			}
			if (ref $category) {
			    # no category selected, take first one
			    $category = $category->[0];
			}
		    }
		    # wrap linked icon in a wrapper for tiling
		    if ($config{webtop}{topdcd}{$category} eq $modname) {
			$icongroup{$category} = $ml->div($link,{class=>"icon"}) . $icongroup{$category};
		    }
		    else {
			$icongroup{$category} .= $ml->div($link,{class=>"icon"});
		    }
		    if ($favourites{$modname}) {
			$icongroup{favourites} .= $ml->div($link,{class=>"icon"});
		    }
		}
	    }
	}
	elsif ($@) {
	    $errors .= $ml->div($@,{class=>"error"});
	}
    }
    if (keys %moduleinfo > 0) {
	# cache module info for better performance, later
	$session{ModuleInfo} = \%moduleinfo;
    }

    my $cat = $config{webtop}{categories};
    my @group = qw(A B);
    my $igroup = 0;

    # optional favourites category
    if (defined $config{webtop}{favourites}) {
	my $title = $config{webtop}{favourites_heading} || "Favourites";
	$ml->_div(
		  $ml->h1($msg{$title}).$icongroup{favourites},
		  { class=>"iconGroupB" }
		  );
    }

    foreach my $category (@$cat) {
	if (! $icongroup{$category}) {
	    $igroup++;
	    next;
	}
	my $title = $config{webtop}{headings}[$igroup] || $category;
	my $gr = $group[$igroup % 2];
	$ml->_div(
		  $ml->h1($msg{$title}).$icongroup{$category},
		  { class=>"iconGroup$gr" }
		  );
	delete $icongroup{$category};
	$igroup++;
    }
    my $igrp = 0;
    foreach my $group (keys %icongroup) {
	next if ($group eq "favourites");
	$igrp++;
	$ml->_div(
		  $ml->h1($msg{$group}).$icongroup{$group},
		  { class=> (($igrp % 2) ? "iconGroupA" : "iconGroupB") }
		  );
	delete $icongroup{group};
    }
    if ($errors && $config{debug}) {
	$ml->Append(
		    $ml->div(
			     &ExSite::HTML::ErrorBox(
						     title=>$msg{"Errors"},
						     pane=>$errors,
						     ),
			     {style=>"padding:20px;" }
			     )
		    );
    }

}
elsif ($webtop_access == -1) {

    # the user has less than administrator access...

    my $dest = $config{auth}{login_destination}[$db->authorize()];
    if ($config{webtop}{deny} eq "login_destination" && 
	$dest !~ /$config{prog}{home}/) {
	$ml->location($dest);
    }
    else {
	# try to show a menu of possible destinations
	require ExSite::Section;

	# 1) find out which websites they are connected to

	my %user = $db->my_user_record;
	my @sites = $db->my_real_sites;

	if (@sites == 0) {
	    $ml->_h3("No Access");
	    $ml->_p("Sorry, but you have not been granted management privileges to any websites.  Please check with your system administrator.");
	    $ml->_p($ml->a("&nbsp;&nbsp;Okay&nbsp;&nbsp;",{href=>"$config{server}{CGIpath}/$config{prog}{logout}", class=>"button"}));
	}
	elsif (@sites == 1) {
	    my $s = new ExSite::Section(section=>$sites[0]);
	    $s->set_context;
	    my $url = $s->get_url;
	    $ml->location($url);
	}
	else {
	    $ml->_h2("$user{name}'s Websites");
	    $ml->_p("You are associated with the following websites. Please select which one you want to open.");
	    my @sitelist;
	    foreach my $site (@sites) {
		my $s = new ExSite::Section(section=>$site);
		$s->set_context;
		my $section = $s->get;
		my $page_id = $s->id("page");
		my $url;
		if ($access == 2) {
		    if ($page_id) {
			$url = "$config{server}{CGIpath}/$config{prog}{page}?_id=$page_id&_cmsmode=1";
			if (@sites == 1) {
			    $ml->location($url);
			}
		    }
		    else {
			$url = "$config{server}{CGIpath}/$config{prog}{ctrlpanel}/CMS?path=/$s->{data}{section_id}";
			if (@sites == 1) {
			    $ml->location($url);
			}
		    }
		}
		else {
		    # no CMS interface for level 1 users
		    $url = $s->httppath;
		}
	
		push @sitelist, { dt=> $ml->a($section->{title},{href=>$url}),
			      dd=> $section->{description} };
	    }
	    $ml->_dl(\@sitelist);
	}
	$ml->__div({style=>"padding:30px;"});
    }
}
else {	
    $ml->Append($share{DB}->login_form());
}

$ml->__div({class=>"webtop"});

# logout action
my $logout = $config{auth}{logout_warning} ?
    &ExSite::HTML::Overlay(
			   pane=>$ml->h1("Log Out").
			   $ml->p("Continuing will log you out of the Administrator screens.").
			   $ml->p(&ExSite::HTML::Button(label=>"Log out",url=>"$config{server}{CGIpath}/$config{prog}{logout}")),
			   label=>$ml->img(undef,{src=>"$config{server}{HTMLpath}/_ExSite/images/button-exit.gif",height=>30,width=>30,border=>0,alt=>"Log out",title=>"Log out"}),
			   height=>150,
			   ) :
    $ml->a(
	   $ml->img(undef,{src=>"$config{server}{HTMLpath}/_ExSite/images/button-exit.gif",height=>30,width=>30,border=>0,alt=>"Log out",title=>"Log out"}),
	   { href=>"$config{server}{CGIpath}/$config{prog}{logout}" }
	   );

my ($section_selector,$section_selector_td);
if ($config{webtop}{section_selector} && $share{DB}->level > 1) {
    my @site = $share{DB}->my_real_sites;
    my $opt = $ml->option($msg{"== choose a site =="},{value=>0});
    my $selected_site = defined $input->{section_id} ? 
	$input->{section_id} :                     # user-selected
	$config{webtop}{section_selector};          # hard-coded
    if ($selected_site =~ /home/) {
	$selected_site = $share{DB}->my_user_record()->{section_id};
    }
    foreach my $site (@site) {
	my $optattr = {value=>$site->{section_id}};
	if ($site->{section_id} == $selected_site) { $optattr->{selected} = undef; }
	$opt .= $ml->option($site->{title},$optattr);
    }
    $section_selector = $ml->form($ml->select($opt,{name=>"section_id",onchange=>"document.forms[0].submit()"}));
    $section_selector_td = $ml->td($section_selector);
}
if ($config{webtop}{section_selector_placement} eq "left") {
    $section_selector_td = undef;
}
else {
    $section_selector = undef;
}

my $topbar = $ml->table(
		     $ml->tr(
			     $ml->td(
				     $ml->a(
					    $ml->img(undef,{src=>"$config{server}{HTMLpath}/_ExSite/images/exsite-button-round.gif",height=>30,width=>30,border=>0,alt=>"Home Page",title=>"Home Page"}),
					    { href=>"$config{server}{server}$config{server}{HTMLpath}", target=>"_top"}
					    )
				     ).
			     $ml->td(
				     $ml->span($config{site}{name},{class=>"title"}).$section_selector,
				     {class=>"titlebar"}
				     ).
			     $section_selector_td.
			     $ml->td(
				     $ml->a(
					    $ml->img(undef,{src=>"$config{server}{HTMLpath}/_ExSite/images/button-home.gif",height=>30,width=>30,border=>0,alt=>"Refresh Webtop",title=>"Refresh Webtop"}),
					    { href=>"$config{server}{CGIpath}/$config{prog}{home}" }
					    )
				     ).
			     $ml->td(
				     $ml->a(
					    $ml->img(undef,{src=>"$config{server}{HTMLpath}/_ExSite/images/button-help.gif",height=>30,width=>30,border=>0,alt=>"Help",title=>"Help"}),
					    { href=>$config{help_url}, target=>"help" }
					    )
				     ).
			     $ml->td($logout)
			     ),
		     {class=>"titlebar"}
		     );

$ml->Prepend($ml->div($topbar,{class=>"topbar"}));
$ml->__body({class=>"webtop"});
$ml->Prepend(
	     $ml->head(
		       $ml->title($config{site}{name}).
		       $ml->meta(undef,{'http-equiv'=>"Content-Type",
					content=>"text/html; charset=$config{charset}"}).
		       $ml->link(undef,{rel=>"shortcut icon",href=>"$config{server}{HTMLpath}/_ExSite/images/exsite.ico"}).
		       $ml->link(undef,{rel=>"stylesheet", type=>"text/css", href=>"$config{server}{HTMLpath}/_ExSite/css/ExSite.css"}).
		       $ml->script(undef,{type=>'text/javascript',src=>"$config{server}{HTMLpath}/_ExSite/js/misc.js"})
		       )
	     );
$ml->__html;

print "Content-type: text/html\n\n";
$ml->Print;
&exsite_close;

# webtop_access
# returns -1 for denial to webtop only
#          0 for complete denial of access
#          1 for access to webtop
# customizable through handler

sub webtop_access {
    # use a handler in cases where simple access-level control is insufficient
    my $stat = $db->run_handler("webtop_access");
    return $stat if (defined $stat);
    # default to access level control
    my $access = $db->level;
    my $min_access = $config{webtop}{access} || 3;
    if ($access) {
	return ($access >= $min_access) ? 1 : -1;
    }
    return 0;
}

### END
