package stdHandlers;
use strict;
use ExSite::Misc;
use Local;

=pod

=head1 stdHandlers.pm

B<stdHandlers.pm> - standard rallycat website localization library.

C<stdHandlers.pm> contains pre-existing library of customizations that general
AMS isntallations of ExSite can make use of:

To install:
add 'use stdHandlers;' to myConfig.pm
add '&std_init($config,$share,$session,$store,$cookie,$msg);' to myConfig::my_exsite_init()
add 'my $skip = [];' to myConfig::my_handlers()
add	'&std_handlers($this,$skip);' to myConfig::my_handlers()
=cut

# Exported Names

BEGIN {
    use Exporter;
    use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
    @ISA         = qw(Exporter);
    @EXPORT_OK   = ();
    %EXPORT_TAGS = ();
    @EXPORT      = qw(
      &std_handlers &std_init
    );

}

use vars ( @EXPORT, @EXPORT_OK );

#=====================================================================
# Non-Exported names:
#
# We cannot import %ExSite::Config::config, because it would create
# a circular reference.  Instead, we just copy it into this module
# using &my_exsite_init().  Ditto for %share.

use vars qw(%config $config $share $session $store $cookie $msg);

#=================================================================
# std_handlers

sub std_init {
	( $config, $share, $session, $store, $cookie, $msg ) = @_;
	%config = %$config;	
}

sub std_handlers {
	my $this = shift;
	my $skip = shift;
	my $db   = $share->{DB};
	if ($db) {
	# install DB-specific handlers here
	}
	if (!grep(/^authenticate$/,@$skip)) {
		$this->handler("authenticate",\&member_pre_authenticate);
	}
	if (!grep(/^is_member$/,@$skip)) {	
		$this->handler("is_member",\&is_member);
	}
	if (!grep(/^log_login$/,@$skip)) {	
		$this->handler("log_login",\&log_login);
	}
	if (!grep(/^page_authorize$/,@$skip)) {	
		$this->handler("page_authorize",\&page_authorize);
	}
	if (!grep(/^Pay_delete_item$/,@$skip)) {	
		$this->handler("Pay_delete_item",\&pay_delete_item);
	}
	if (!grep(/^Pay_init$/,@$skip)) {	
		$this->handler("Pay_init",\&initialize_cart);
	}
	if (!grep(/^Pay_payment_by_invoice$/,@$skip)) {	
		$this->handler("Pay_payment_by_invoice",\&pay_by_invoice);
	}
	if (!grep(/^Pay_receipt_finalize$/,@$skip)) {	
		$this->handler("Pay_receipt_finalize",\&pay_receipt_finalize);
	}
	if (!grep(/^receivable_activate$/,@$skip)) {	
		$this->handler("receivable_activate",\&activate_invoice);
	}
	if (!grep(/^select_foreign_key$/,@$skip)) {	
		$this->handler("select_foreign_key",\&select_foreign_key);    
	}
	if (!grep(/^user_owns$/,@$skip)) {	
		$this->handler("user_owns",\&user_owns);
	}
	if (!grep(/^validate_cart$/,@$skip)) {	
		$this->handler("validate_cart",\&validate_cart);	
	}
	if (!grep(/^get_custom_contact_list$/,@$skip)) {	
		$this->handler("get_custom_contact_list",\&custom_contact_list);
	}
	if (!grep(/^custom_contact_list_add_meta$/,@$skip)) {	
		$this->handler("custom_contact_list_add_meta",\&list_add_meta);
	}
	if (!grep(/^custom_contact_list_match_hash$/,@$skip)) {	
		$this->handler("custom_contact_list_match_hash",\&list_match_hash);
	}
	if (!grep(/^export_custom$/,@$skip)) {
		$this->handler("export_custom",\&export_custom);	
	}
	if ($config{stdHandler}{item_location} && !grep(/^ReceivableItem_location$/,@$skip)) {
		$this->handler("ReceivableItem_location",\&item_location);	
	}	
}

#=================================================================
# required rallycat handlers

sub member_pre_authenticate {
	my $this = shift;
	if ( exists $this->{authenticated} ) {
		return $this->{authenticated};
	}

	# authenticate the user if they are in the middle of registering as a user
	if ( $share->{use_prelim_identity} && keys %{ $session->{prelim_identity} } ) {
		if ( $share->{use_prelim_identity} == $session->{prelim_identity}{uid} ) {
			$share->{identity} = $session->{prelim_identity};
		}
		$this->{authenticated} = 1;
		return 1;
	}
	return undef;
}

# we use the association membership definition to control permissions
sub is_member {
	my ( $this, $site ) = @_;
	return 1 if ( $share->{DB}->is_manager($site) );
	require Modules::Membership::Member;
	my $db = $share->{DB};
	return undef if ( !$db->level );
	my $member = new Modules::Membership::Member( id => $db->my_uid );
	return $member->is_valid_member;
}

sub log_login {
	my $db   = shift;
	# don't log if using Membership::login()
	return undef if ( $session->{admin_uid} || $session->{key_contact_uid} );
	my $user = $db->my_user_record();
	if ( $user->{member_id} ) {
		$user->{login_count}++;
		$user->{last_login} = undef;
		$db->update( "member", $user );
	}
}

sub page_authorize {
	my $this = shift;

	# skip authcheck if we are publishing, since the page is public by nature
	return 1 if ( $this->is_publishing );
	# sysadmins can view anything from anywhere
	my $db = $share->{DB};
	return 1 if ( $db->is_admin );
	# regular admins can view pages from their own sites from any URL
	return 1 if ( $db->is_manager( $this->get("section") ) );
	# non-admins can only view active pages
	return 0 if ( !$this->is_active() );
	# check for a valid page URL
	return 0 if ( $config{cms}{valid_page_check} && !$this->is_valid );

	# guest pages
	if ( ( ref $this ) =~ /Page/ ) {
		if ( !$this->is_public() ) {
		    my $db = $share->{DB};
		    if ( $db->is_level_one ) {
		        if ( !$db->is_member ) {
		            my %allowpage = map { $_ => 1 } @{ $config->{guest_pages} };
		            if ( !$allowpage{ $this->id } ) {
		                $this->error(
		                    $msg->{
		                    $config{stdHandler}{page_deny_message} ||
	"You are logged in but this page is restricted to valid members only. Please upgrade/renew your membership first."
		                    }
		                );
		                return 0;
		            }
		            else {
		                return 1;
		            }
		        }
		    }
		}
	}

	# check access permissions for regular members
	require ExSite::Module;
	my $stat = eval "require Modules::Permissions";
	if ($stat) {
		&ExSite::Module::get_module("Permissions");
		my $stat = &Modules::Permissions::allow($this);
		return $stat if ( defined $stat );
	}
	else {
		my $pdata  = $this->get();
		my $access = $pdata->{access};
		return 1 if ( $access eq "public" || !defined $access );
		return 1 if ( $pdata->{access} =~ /member/
		    && $db->is_member( $this->id("section") ) );
	}

	# non-public, non-member page, and we are not an admin
	return 0;
}

sub pay_delete_item {
	my ( $this, $item ) = @_;
	my $type = $item->getdata("objtype");
	my $id   = $item->getdata("objid");
	if ( $type && $id ) {
		if ( $type eq "evt_reg" ) {
		    require ExSite::Module;
		    &ExSite::Module::get_module("EvtReg");
		    return &Modules::EvtReg::delete_cart_registration( $this, $item );
		}
		if ( $type eq "member" ) {
			require ExSite::Module;
			&ExSite::Module::get_module("Membership");
			return &Modules::Membership::delete_cart_membership( $this, $item );
		}
	}
}	

sub initialize_cart {
	my $this = shift;

	# let the authentication system know that it's okay to make updates
	# to this user's account data (since the user has not yet been fully
	# activated and does not have permission to update their data).
	if ( keys %{ $session->{prelim_identity} } ) {
		$share->{identity}            = $session->{prelim_identity};
		$share->{use_prelim_identity} = $share->{identity}{uid};
	}
	return undef;
}

sub pay_by_invoice {
	my ($this) = @_;
	my $out;
	my $db = $share->{DB};
	if ( $this->{invoice} ) {
		my $items         = $this->{invoice}->loaditems();
		my $receivable_id = $this->{invoice}->id;
		while ( my $item = $items->next ) {
			my $type = $item->getdata("objtype");
			my $id   = $item->getdata("objid");
			my $note = $item->getdata("note");
			if ( $type eq "member" ) {
				require ExSite::Module;
				my $m = &ExSite::Module::get_module("Membership");
				if ( $m->setup_member($id) ) {
					if ( $note =~ /^Renew/ ) {
						$out .= $m->renew_member($receivable_id);
					} elsif ( $note =~ /^New/ ) {
						$out .= $m->finalize_application($receivable_id);
					}
				}
			}
		}
	}
	return $out;
}

sub pay_receipt_finalize {
	my ( $this, $receivable_id ) = @_;
	my $out;
	my $db = $share->{DB};
	my @receivable_items = $db->fetch_child( "receivable_item", "receivable", $receivable_id );
	foreach my $item (@receivable_items) {
		if ( $item->{objtype} && $item->{objid} ) {
		    if ( $item->{objtype} eq "member" ) {
		        require ExSite::Module;
		        my $m = &ExSite::Module::get_module("Membership");
		        if ( $m->setup_member( $item->{objid} ) ) {
		            if ( $item->{note} =~ /^Renew/ ) {
		                $out .= $m->renew_member($receivable_id);
                    } elsif ( $item->{note} =~ /^Upgrade/ ) {
					 	$out .= $m->upgrade_member($receivable_id);
		            } else {
		                $out .= $m->finalize_application($receivable_id);
		            }
		        }
		    }
		    elsif ( $item->{objtype} eq "evt_reg" ) {
		        require ExSite::Module;
		        &ExSite::Module::get_module("EvtReg");
		        &Modules::EvtReg::finalize_registration( $this, $receivable_id );
		    }
		}
	}
    return $out;
}

sub activate_invoice {

    # set registrations to pending
    my $this  = shift;
    my $db    = $share->{DB};
    my $items = $this->loaditems();

    while ( my $item = $items->next ) {
        if ( $item->getdata("objtype") eq "evt_reg" ) {
            my $mod = ExSite::Module::get_module("EvtReg");
            $mod->activate_invoice( $this, 1 );
            last;
        }
    }

    return undef;    # also executes regular logic
}

sub select_foreign_key {
	my ( $this, $table, $record, $column, $ftable ) = @_;
	if ( $table eq "member" and $column eq "parent_id" ) {
		my $rdata = $this->fetch_match( "member", { parent_id => 0, type => "member%" }, ["organization"] );
		return $rdata;
	}
	return undef;
}

# special record ownership logic
sub user_owns {
	my ( $this, $table, $record ) = @_;
	my $db = $share->{DB};
	my ($row_id,$data);
	my $level = $db->authorize;

	if (ref $record) {
		$row_id = $record->{$this->get_key($table)};
		$data = $record;
	} else {
		$row_id = $record;
		$data = $db->fetch( $table, $record );
	}
	
	if ( $table eq "receivable" && $row_id == $session->{invoice} ) {
		return 1;
	}
	elsif ( $table eq "account" ) {
		if ( $data->{member_id} == $db->my_uid() || $data->{member_id} == 0 ) {
			return 1;
		}
	}
	elsif ( $table =~ /^attribute$/ ) {
		return $share->{DB}->user_owns($data->{tablename},$data->{id});
	}
	elsif ( $table eq "acctcode" ) {
		return 1 if ( $level == 2 );
	}
	elsif ( $table eq "surcharge" ) {
		my $sid = $data->{section_id};
		return 1 if ( $level == 2 && !$sid );
	}
	# permissions for related members
	elsif ( $table eq "member" ) {
		my $uid   = $db->my_uid;
		my $u_pid = $db->my_user_record->{parent_id};
		my $mid   = $data->{member_id};
		my $pid   = $data->{parent_id};
		if ( $u_pid && $pid == $u_pid ) {
			return 1;	# sibling
		}
		elsif ( $uid == $pid ) {
			return 1;	# child
		}
		elsif ( $mid == $u_pid ) {
			return 1;	# parent
		}
	}
	return undef;
}

sub validate_cart {
    my ($this,$final) = @_;
    if ( eval 'require Modules::EvtReg' ) {
        return &Modules::EvtReg::validate_cart($this,$final);
    }
    return undef;
}

# standard contact list handler
sub custom_contact_list {
    my ( $this, $match ) = @_;
    if ( my $rid = $match->{member_report_id} ) {
        my $m = &ExSite::Module::get_module("MembershipReports");
        $m->load_report( rid => $rid, format => "ContactList" );
        my @c = $m->show_report();
        return wantarray ? @c : \@c;
    }
    elsif ($match->{wnr_contact_list}) {
	my $wnr = &ExSite::Module::get_module("WhosNotRegistered");
	return $wnr->ctrl_panel($match);
    }
    elsif ( my $hid = $match->{member_report_history_id} ) {
        my $m = &ExSite::Module::get_module("MembershipReports");
        my $h = $share->{DB}->fetch( "member_report_history", $hid );
        require JSON;
        my $title   = $h->{title};
        my $headers = JSON::jsonToObj( $h->{headers} );
        my $data    = JSON::jsonToObj( $h->{data} );
        my $r       = new ExSite::ReportBuilder( title => $title, headers => $headers, data => $data );
        return $r->records();
    }
}

sub list_add_meta {
    my ( $this, $c ) = @_;
    $c->{_alpha} = uc $c->{name};
    $c->{_name}  = $c->{name};
}

sub list_match_hash {
    my ( $this, $in, $match ) = @_;
    foreach my $key ( keys %$in ) {
        $match->{$key} = $in->{$key};
    }
}

sub item_location {
        my ($this) = shift;
        my $is_evt_reg = ($this->getdata("objtype") eq "evt_reg") || 0;
        my $is_surcharge = (!$this->getdata("objtype") && $this->getdata("surcharge_id") > 0 && $this->getdata("acctcode_id") > 0) || 0;
        if ($is_evt_reg || $is_surcharge){
                my $objid = 0;
                # evt_reg
                if ($is_evt_reg){
                        $objid = $this->getdata("objid");
                # surcharge
                #
                # If the receivable item has a surcharge_id>0,
                # look for other receivable items on the invoice that have the same accounting code and objtype ='evt_reg' and cost>0
                # if any of those evt_reg receivable items are for an event with a location, then return that location's [province,country]
                }elsif ($is_surcharge){
                        my @recv = $share->{DB}->custom_query("select * from receivable_item where acctcode_id = ? and objtype = 'evt_reg' and cost > 0 and receivable_id=?",$this->getdata("acctcode_id"), $this->getdata("receivable_id"));
                        $objid = scalar(@recv) > 0 ? $recv[0]{objid} : 0;
                }
                if ($objid){
                        # load EvtReg frameworks
                        &ExSite::Module::get_module("EvtReg");
                        my $reg = new Modules::Registration::Registration(id=>$objid);
                        my $evt = $reg->event();
                        if ($evt) {
                                my $locs = $evt->get_locations();
                                if ($locs->count) {
                                        my @contact = $share->{DB}->fetch_child("contact","location",$locs->first->getdata("location_id"));
                                        if (scalar @contact) {
                                                return [$contact[0]{provstate},$contact[0]{country}];
                                        }
                                }
                                else {
                                    my $main = $evt->main_event();
                                    if ($main->id != $evt->id) {
                                        $locs = $main->get_locations();
                                        if ($locs->count) {
                                            my @contact = $share->{DB}->fetch_child("contact","location",$locs->first->getdata("location_id"));
                                            if (scalar @contact) {
                                                return [$contact[0]{provstate},$contact[0]{country}];
                                            }
                                        }
                                    }
                                }
                        }
                }
        }
        return undef;
}

# minage = minage in days before report is saved again
sub export_custom {
    my ($this, %opt) = @_;
    require ExSite::Object;
    require JSON;
    my $title   = $this->{title};
    my @old = $share->{DB}->fetch_match("member_report_history",{title=>$title});
    foreach my $h (@old) {
        my $t = new ExSite::Time($h->{ctime}, "sql_datetime");
        $t->add($opt{minage} || 1,"days");
        return undef if ($t > new ExSite::Time);
    }
    my $headers = $this->{headers};
    my $data    = $this->{data};
    my $total   = (ref $data eq "ARRAY") ? scalar @$data : 0;
    my $date    = new ExSite::Time->write("date");
    my %history = (
        title            => $title,
        headers          => JSON::to_json($headers),
        data             => JSON::to_json($data),
        total            => $total
    );
    my $h = new ExSite::Object(type => "member_report_history", data => \%history);
    my $hid = $h->force_save();
    return $hid;
}

1;
